@@ -162,3 +162,27 @@ class SceneAbbreviatedFilenameError(UpsiesError):
     """
     def __init__(self, file_name):
         super().__init__(f'Abbreviated scene file name is verboten: {file_name}')
+
+
+class FoundDupeError(RequestError):
+    """
+    Tried to upload files that have already been uploaded
+
+    :param filenames: Sequence of file names that have already been uploaded
+    """
+    def __init__(self, filenames):
+
+        if not filenames:
+            msg = ['Potential duplicate files found']
+
+        else:
+            files_count = len(filenames)
+            if files_count != 1:
+                msg = [f'{files_count} potential duplicate files found']
+            else:
+                msg = ['1 potential duplicate file found']
+
+            msg.append(':\n - ')
+            msg.append('\n - '.join(filenames))
+
+        super().__init__(''.join(msg))
