@@ -97,8 +97,13 @@ class JobBase(abc.ABC):
         is supposed to be evaluated by any job starting facilities.
 
         If this value is falsy, :meth:`start` must be called manually.
+
+        See also :attr:`prejobs`.
         """
-        return self._autostart
+        return (
+            self._autostart
+            and all(prejob.is_finished for prejob in self.prejobs)
+        )
 
     @property
     def condition(self):
@@ -162,15 +167,28 @@ class JobBase(abc.ABC):
         """
         return self._signal
 
+    @property
+    def prejobs(self):
+        """
+        Sequence of prerequisite :class:`jobs <upsies.jobs.base.JobBase>`
+
+        :attr:`autostart` stays `False` until all `prejobs` have finished.
+        """
+        return self._prejobs
+
+    @prejobs.setter
+    def prejobs(self, prejobs):
+        self._prejobs = tuple(prejobs)
+
     def __init__(self, *, home_directory=None, cache_directory=None,
                  ignore_cache=False, hidden=False, autostart=True,
-                 condition=None, callbacks={}, **kwargs):
+                 condition=None, prejobs=(), callbacks={}, **kwargs):
         self._home_directory = home_directory if home_directory else ''
         self._cache_directory = cache_directory if cache_directory else constants.DEFAULT_CACHE_DIRECTORY
         self._ignore_cache = bool(ignore_cache)
         self._hidden = bool(hidden)
         self._autostart = bool(autostart)
-        self.condition = condition or (lambda: True)
+        self.condition = condition if condition is not None else (lambda: True)
         self._is_started = False
         self._is_executed = False
         self._exception = None
@@ -180,6 +198,7 @@ class JobBase(abc.ABC):
         self._info = ''
         self._tasks = []
         self._finished_event = asyncio.Event()
+        self.prejobs = prejobs
 
         self._signal = signal.Signal('output', 'info', 'warning', 'error', 'finished')
         self._signal.register('output', lambda output: self._output.append(str(output)))
@@ -469,10 +488,8 @@ class JobBase(abc.ABC):
                     f.write(emissions_serialized)
                     f.write(b'\n')
             except OSError as e:
-                if e.strerror:
-                    raise RuntimeError(f'Unable to write cache {self.cache_file}: {e.strerror}')
-                else:
-                    raise RuntimeError(f'Unable to write cache {self.cache_file}: {e}')
+                msg = e.strerror if e.strerror else str(e)
+                raise RuntimeError(f'Unable to write cache {self.cache_file}: {msg}')
 
     def _read_cache(self):
         """
@@ -489,10 +506,8 @@ class JobBase(abc.ABC):
                 with open(self.cache_file, 'rb') as f:
                     emissions_serialized = f.read()
             except OSError as e:
-                if e.strerror:
-                    raise RuntimeError(f'Unable to read cache {self.cache_file}: {e.strerror}')
-                else:
-                    raise RuntimeError(f'Unable to read cache {self.cache_file}: {e}')
+                msg = e.strerror if e.strerror else str(e)
+                raise RuntimeError(f'Unable to read cache {self.cache_file}: {msg}')
             else:
                 emissions_deserialized = self._deserialize_from_cache(emissions_serialized)
                 _log.debug('%s: Replaying cached signals: %r', self.name, emissions_deserialized)
