@@ -25,9 +25,8 @@ class CustomJob(JobBase):
 
         :param name: Name for internal use
         :param label: Name for user-facing use
-        :param worker: Coroutine function (a function defined with an
-            ``async def`` syntax) that takes the job instance as a
-            positional argument.
+        :param worker: Coroutine function that takes the job instance as a
+            positional argument
         :param catch: Sequence of :class:`Exception` classes to catch from
             `worker` and pass to :meth:`~.JobBase.error`
 
