@@ -3,8 +3,9 @@ Get information from the user
 """
 
 import collections
+import re
 
-from .. import errors, utils
+from .. import utils
 from . import JobBase
 
 import logging  # isort:skip
@@ -19,14 +20,10 @@ class ChoiceJob(JobBase):
     attribute:
 
         ``dialog_updated``
-            Emitted when the :attr:`choices` or :attr:`focused` properties are
-            set or when :attr:`choices` is modified. Registered callbacks get
-            the job instance as a positional argument.
-
-        ``chosen``
-            Emitted when the :attr:`choice` property is set or when the job's
-            cache is read. Registered callbacks get :attr:`choice` as a
-            positional argument.
+            Emitted when the :attr:`choices`, :attr:`focused` or
+            :attr:`autodetected` properties are set or when :attr:`choices` is
+            modified. Registered callbacks get the job instance as a positional
+            argument.
     """
 
     @property
@@ -44,8 +41,7 @@ class ChoiceJob(JobBase):
 
         A choice is a :class:`tuple` with 2 items. The first item is a
         human-readable :class:`str` and the second item is any object that is
-        emitted via the ``chosen`` signal and made available as :attr:`choice`
-        when the job is finished.
+        available as :attr:`choice` when the job is finished.
 
         Choices may also be passed as a flat iterable of :class:`str`, in which
         case both items in the tuple are identical.
@@ -83,7 +79,8 @@ class ChoiceJob(JobBase):
             callback=lambda _: self.signal.emit('dialog_updated', self),
         )
 
-        # Try to restore focus
+        # Try to restore focus if the previously focused item still exists,
+        # default to first choice
         self._focused_index = 0
         if prev_focused:
             prev_label, prev_value = prev_focused
@@ -94,93 +91,154 @@ class ChoiceJob(JobBase):
 
         self.signal.emit('dialog_updated', self)
 
-    @property
-    def focused(self):
+    def get_index(self, thing):
         """
-        Currently focused choice (2-tuple)
+        Return index in :attr:`choices`
 
-        Setting this property to `None` focuses the first choice in
-        :attr:`choices`.
+        :param thing: Identifier of a choice in :attr:`choices`
 
-        Setting this property to an :class:`int` focuses the choice at that
-        index in :attr:`choices`.
+            This can be:
 
-        Setting this property to any other value assumes that value is a choice
-        in :attr:`choices` or one of the values of one of the :attr:`choices`.
-        If that fails, :class:`ValueError` is raised.
+            `None`
+                Return `None`
+
+            an index (:class:`int`)
+               Return `thing`, but limited to the minimum/maximum valid index.
+
+            one of the 2-tuples in :attr:`choices`
+                Return the index of `thing` in :attr:`choices`.
+
+            an item of one of the 2-tuples in :attr:`choices`
+                Return the index of the first 2-tuple in :attr:`choices` that
+                contains `thing`.
+
+            a :func:`regular expression <re.compile>`
+                Return the index of the first 2-tuple in :attr:`choices` that
+                contains something that matches `thing`. Non-string values are
+                converted to :class:`str` for matching against the regular
+                expression.
+
+        :raise ValueError: if `thing` is not found in :attr:`choices`
         """
-        focused_index = getattr(self, '_focused_index', None)
-        if focused_index is not None:
-            return self.choices[focused_index]
+        if thing is None:
+            return None
+
+        elif isinstance(thing, int):
+            return max(0, min(thing, len(self.choices) - 1))
+
+        elif thing in self.choices:
+            return self.choices.index(thing)
 
-    @focused.setter
-    def focused(self, focused):
-        if focused is None:
-            self._focused_index = 0
-        elif isinstance(focused, int):
-            index = max(0, min(focused, len(self.choices) - 1))
-            self._focused_index = index
-        elif focused in self.choices:
-            self._focused_index = self.choices.index(focused)
         else:
-            for i, choice in enumerate(self.choices):
-                if focused == choice[0] or focused == choice[1]:
-                    self._focused_index = i
-                    break
-            # The else clause is evaluated if the for loop ends without reaching
-            # `break`.
-            else:
-                raise ValueError(f'Invalid choice: {focused!r}')
+            for i, (label, value) in enumerate(self.choices):
+                # Focus by human-readable text or value
+                if thing == label or thing == value:
+                    return i
 
-        self.signal.emit('dialog_updated', self)
+                # Focus by matching regex against human-readable text or value
+                elif isinstance(thing, re.Pattern):
+                    value_str = str(value)
+                    if thing.search(label) or thing.search(value_str):
+                        return i
+
+        raise ValueError(f'No such choice: {thing!r}')
+
+    def get_choice(self, thing):
+        """
+        Return item in :attr:`choices`
+
+        :param thing: See :meth:`get_index`
+
+        If `thing` is `None`, return the currently focused choice, which is
+        indicated by :attr:`focused_index`.
+        """
+        choice_index = self.get_index(thing)
+        if choice_index is None:
+            choice_index = self.focused_index
+        return self.choices[choice_index]
 
     @property
     def focused_index(self):
         """Index of currently focused choice in :attr:`choices`"""
         return getattr(self, '_focused_index', None)
 
+    @property
+    def focused(self):
+        """
+        Currently focused choice (2-tuple)
+
+        This property can be set to anything that is a valid value for
+        :meth:`get_index`.
+        """
+        focused_index = self.focused_index
+        if focused_index is not None:
+            return self.choices[focused_index]
+
+    @focused.setter
+    def focused(self, focused):
+        # focused_index can't be set to None, so we default to first choice
+        self._focused_index = self.get_index(focused)
+        if self._focused_index is None:
+            self._focused_index = 0
+        self.signal.emit('dialog_updated', self)
+
+    @property
+    def autodetected_index(self):
+        """Index of autodetected choice in :attr:`choices` or `None`"""
+        return getattr(self, '_autodetected_index', None)
+
+    @property
+    def autodetected(self):
+        """
+        Autodetected choice (2-tuple) or `None`
+
+        This property can be set to anything that is a valid value for
+        :meth:`get_index`.
+        """
+        autodetected_index = self.autodetected_index
+        if autodetected_index is not None:
+            return self.choices[autodetected_index]
+
+    @autodetected.setter
+    def autodetected(self, autodetected):
+        self._autodetected_index = self.get_index(autodetected)
+        self.signal.emit('dialog_updated', self)
+
     @property
     def choice(self):
         """
         User-chosen value if job is finished, `None` otherwise
 
-        Setting this property finishes the job.
-
         While :attr:`~.base.JobBase.output` contains the user-readable string
-        (first item of the chosen item in :attr:`choices`), this is the object
-        attached to it (second item).
-
-        This property can be set to an index in :attr:`choices`, an item in
-        :attr:`choices` or the first or second item of an item in
-        :attr:`choices`.
+        (first item of the chosen 2-tuple in :attr:`choices`), this is the
+        object attached to it (second item).
         """
-        return getattr(self, '_choice', None)
+        choice = getattr(self, '_choice', None)
+        if choice is not None:
+            return choice
 
-    @choice.setter
-    def choice(self, choice):
-        def choose(choice):
-            # Machine-readable choice
-            self.signal.emit('chosen', choice[1])
-            # User-readable string
-            self.send(choice[0])
-            self.finish()
-
-        if choice in self.choices:
-            choose(choice)
-        elif isinstance(choice, int):
-            try:
-                choose(self.choices[choice])
-            except IndexError:
-                raise ValueError(f'Invalid choice: {choice!r}')
-        elif isinstance(choice, str):
-            for c in self.choices:
-                if choice == c[0] or choice == c[1]:
-                    choose(c)
-                    break
-            else:
-                raise ValueError(f'Invalid choice: {choice!r}')
+    def _set_choice(self, choice):
+        # This method is called via `output` signal (see initialize()), which is
+        # emitted by send(), so make_choice() doesn't need to call _set_choice().
+        if hasattr(self, '_choice'):
+            raise RuntimeError(f'{self.name}: Choice was already made: {self.choice}')
         else:
-            raise ValueError(f'Invalid choice: {choice!r}')
+            label, value = self.get_choice(choice)
+            self._choice = value
+
+    def make_choice(self, thing):
+        """
+        Make a choice and :meth:`~.JobBase.finish` this job
+
+        :param thing: See :meth:`get_choice`
+
+        After this method is called, this job :attr:`~.JobBase.is_finished`,
+        :attr:`~.JobBase.output` contains the human-readable label of the choice
+        and :attr:`choice` is the machine-readable value of the choice.
+        """
+        label, value = self.get_choice(thing)
+        self.send(label)
+        self.finish()
 
     def set_label(self, identifier, new_label):
         """
@@ -201,7 +259,8 @@ class ChoiceJob(JobBase):
         if self.choices != new_choices:
             self.choices = new_choices
 
-    def initialize(self, *, name, label, choices, focused=None):
+    def initialize(self, *, name, label, choices, focused=None, autodetected=None,
+                   autodetect=None, autodetect_and_finish=None):
         """
         Set internal state
 
@@ -209,18 +268,49 @@ class ChoiceJob(JobBase):
         :param label: Name for user-facing use
         :param choices: Iterable of choices the user can make
         :param focused: See :attr:`focused`
+        :param autodetected: See :attr:`autodetected`
+        :param autodetect: Callable that sets :attr:`autodetected` in
+            :meth:`.JobBase.start`
+
+            `autodetect` gets the job instance (``self``) as a positional
+            argument.
+        :param autodetect_and_finish: Same as `autodetect`, but :meth:`finish`
+            this job if the returned value is not `None`
 
         :raise ValueError: if `choices` is shorter than 2 or `focused` is
             invalid
         """
         self._name = str(name)
         self._label = str(label)
+        self._autodetect = autodetect
+        self._autodetect_and_finish = autodetect_and_finish
+
         self.signal.add('dialog_updated')
-        self.signal.add('chosen')
-        self.signal.record('chosen')
-        self.signal.register('chosen', lambda choice: setattr(self, '_choice', choice))
+        self.signal.register('output', self._set_choice)
+
         self.choices = choices
-        self.focused = focused
+        self.autodetected = autodetected
+
+        if focused is not None:
+            self.focused = focused
+        else:
+            self.focused = autodetected
+
+    def start(self):
+        super().start()
+
+        # Job is finished right away if output was cached previously
+        if not self.is_finished:
+            if self._autodetect:
+                autodetected = self._autodetect(self)
+                if autodetected is not None:
+                    self.autodetected = self.focused = autodetected
+
+            if self._autodetect_and_finish:
+                autodetected = self._autodetect_and_finish(self)
+                if autodetected is not None:
+                    self.autodetected = self.focused = autodetected
+                    self.make_choice(self.focused)
 
 
 class TextFieldJob(JobBase):
@@ -230,17 +320,21 @@ class TextFieldJob(JobBase):
     This job adds the following signals to the :attr:`~.JobBase.signal`
     attribute:
 
-        ``dialog_updating``
-            Emitted when :meth:`fetch_text`` is called to indicate activity.
-            Registered callbacks get the job instance as a positional argument.
+        ``text``
+            Emitted when :attr:`text` was changed without user input. Registered
+            callbacks get the new text as a positional argument.
 
-        ``dialog_updated``
-            Emitted when any property is set. Registered callbacks get the job
-            instance as a positional argument.
+        ``is_loading``
+            Emitted when :attr:`is_loading` was changed. Registered callbacks
+            get the new :attr:`is_loading` value as a positional argument.
 
-        ``accepted``
-            Emitted when the user accepts the text. Registered callbacks get the
-            entered text as a positional argument.
+        ``read_only``
+            Emitted when :attr:`read_only` was changed. Registered callbacks get
+            the new :attr:`read_only` value as a positional argument.
+
+        ``obscured``
+            Emitted when :attr:`obscured` was changed. Registered callbacks get
+            the new :attr:`obscured` value as a positional argument.
     """
 
     @property
@@ -259,17 +353,21 @@ class TextFieldJob(JobBase):
     @text.setter
     def text(self, text):
         self._text = self._normalizer(str(text))
-        self.signal.emit('dialog_updated', self)
+        self.signal.emit('text', self.text)
 
     @property
     def obscured(self):
-        """Whether the text is unreadable, e.g. when entering passwords"""
+        """
+        Whether the text is unreadable, e.g. when entering passwords
+
+        This is currently not fully implemented.
+        """
         return getattr(self, '_obscured', False)
 
     @obscured.setter
     def obscured(self, obscured):
         self._obscured = bool(obscured)
-        self.signal.emit('dialog_updated', self)
+        self.signal.emit('obscured', self.obscured)
 
     @property
     def read_only(self):
@@ -279,7 +377,17 @@ class TextFieldJob(JobBase):
     @read_only.setter
     def read_only(self, read_only):
         self._read_only = bool(read_only)
-        self.signal.emit('dialog_updated', self)
+        self.signal.emit('read_only', self.read_only)
+
+    @property
+    def is_loading(self):
+        """Whether :attr:`text` is currently being changed automatically"""
+        return getattr(self, '_is_loading', False)
+
+    @is_loading.setter
+    def is_loading(self, is_loading):
+        self._is_loading = bool(is_loading)
+        self.signal.emit('is_loading', self.is_loading)
 
     @property
     def exit_code(self):
@@ -307,7 +415,7 @@ class TextFieldJob(JobBase):
             is called before `validator`. It should not raise any exceptions.
         :param validator: callable or None
         :param bool obscured: Whether :attr:`obscured` is set to `True`
-            initially
+            initially (currently not working)
         :param bool read_only: Whether :attr:`read_only` is set to `True`
             initially
         """
@@ -315,9 +423,10 @@ class TextFieldJob(JobBase):
         self._label = str(label)
         self._validator = validator or (lambda _: None)
         self._normalizer = normalizer or (lambda text: text)
-        self.signal.add('dialog_updating')
-        self.signal.add('dialog_updated')
-        self.signal.add('accepted')
+        self.signal.add('text')
+        self.signal.add('is_loading')
+        self.signal.add('read_only')
+        self.signal.add('obscured')
         self.text = text
         self.obscured = obscured
         self.read_only = read_only
@@ -339,38 +448,37 @@ class TextFieldJob(JobBase):
             super().send(output)
             super().finish()
 
-    async def fetch_text(self, coro, default_text=None, finish_on_success=False, error_is_fatal=False):
+    async def fetch_text(self, coro, default_text=None, finish_on_success=False, nonfatal_exceptions=()):
         """
         Get :attr:`text` from coroutine
 
         :param coro: Coroutine that returns the new :attr:`text` or `None` to
             use `default_text`
         :param default_text: String to use if `coro` raises
-            :class:`~.errors.RequestError` or if `coro` returns `None`
+            `nonfatal_exceptions` or if `coro` returns `None`
         :param finish_on_success: Whether to call :meth:`finish` after setting
             :attr:`text` to `coro` return value
-        :param error_is_fatal: Whether to call :meth:`error` and :meth:`finish`
-            if `coro` raises :class:`~.errors.RequestError` instead of
-            :meth:`warn`
+        :param nonfatal_exceptions: Sequence of exception classes that may be
+            raised by `coro` without indicating failure
+
+            Non-fatal exceptions are passed to :meth:`warn` while fatal
+            exceptions are raised.
         """
-        self.read_only = True
-        self.signal.emit('dialog_updating', self)
+        self.is_loading = self.read_only = True
 
         try:
-            # Try to set text field value, e.g. from IMDb
             text = await coro
 
-        except errors.RequestError as e:
+        except Exception as e:
             if default_text is not None:
                 self.text = default_text
 
-            if error_is_fatal:
-                # Finish the job with an error status
-                self.error(e)
-            else:
-                # Inform user about failure and allow them to make manual
-                # adjustments to the default text
+            if isinstance(e, nonfatal_exceptions):
+                # Display error and allow user to adjust the default text
                 self.warn(e)
+            else:
+                self.finish()
+                raise e
 
         else:
             if text is not None:
@@ -385,4 +493,4 @@ class TextFieldJob(JobBase):
 
         finally:
             # Always re-enable text field after we're done messing with it
-            self.read_only = False
+            self.is_loading = self.read_only = False
