@@ -171,7 +171,7 @@ def _normalize_timestamps(video_file, timestamps, count):
     Return list of validated human-readable timestamps
 
     :params video_file: Path to video file
-    :parms timestamps: Sequence of arguments for :func:`~utils.timestamp.parse`
+    :parms timestamps: Sequence of arguments for :func:`.utils.timestamp.parse`
     :parms int count: Desired number of timestamps
 
     :raise ValueError: if an item in `timestamps` is invalid
