@@ -143,8 +143,8 @@ class SubmitJob(JobBase):
         """
         Sequence of jobs to do before submission
 
-        This is the same as :attr:`~TrackerJobsBase.jobs_before_upload` but with
-        all `None` values and disabled jobs filtered out.
+        This is the same as :attr:`.TrackerJobsBase.jobs_before_upload` but
+        with all `None` values and disabled jobs filtered out.
         """
         return tuple(job for job in self._tracker_jobs.jobs_before_upload
                      if job and job.is_enabled)
@@ -154,7 +154,7 @@ class SubmitJob(JobBase):
         """
         Sequence of jobs to do after successful submission
 
-        This is the same as :attr:`~TrackerJobsBase.jobs_before_upload` but with
+        This is the same as :attr:`.TrackerJobsBase.jobs_after_upload` but with
         all `None` values and disabled jobs filtered out.
         """
         return tuple(job for job in self._tracker_jobs.jobs_after_upload
