@@ -5,10 +5,9 @@ Create torrent file
 import fnmatch
 import os
 import queue
-import time
 
 from .. import errors
-from ..utils import btclients, daemon, fs, is_regex_pattern, torrent
+from ..utils import daemon, fs, is_regex_pattern, torrent
 from . import base
 
 import logging  # isort:skip
@@ -37,7 +36,8 @@ class CreateTorrentJob(base.JobBase):
         ``progress_update``
             Emitted at roughly equal intervals to provide information about the
             torrent creation progress. Registered callbacks get a
-            :class:`~.torrent.CreateTorrentProgress` instance as a positional
+            :class:`~.torrent.CreateTorrentProgress` or
+            :class:`~.torrent.FindTorrentProgress` instance as a positional
             argument.
     """
 
@@ -78,20 +78,33 @@ class CreateTorrentJob(base.JobBase):
         self.signal.add('file_tree')
         self.signal.add('progress_update')
         self._torrent_process = None
+        self._activity = ''
+
+    @property
+    def activity(self):
+        """
+        What is currently being done
+
+        ``"hashing"``
+            If we are hashing files.
+
+        ``"searching"``
+            If we are searching for a torrent we can copy hashes from.
+
+        ``""``
+            None of the above is happening (e.g. we are getting the file tree).
+        """
+        return self._activity
 
     def execute(self):
         """Get announce URL from `tracker`, then execute torrent creation subprocess"""
-        if not self.ignore_cache and os.path.exists(self._torrent_path):
-            self._handle_torrent_created(self._torrent_path)
-            self.finish()
-        else:
-            self._get_announce_url_task = self.add_task(self._get_announce_url())
-            self._get_announce_url_task.add_done_callback(
-                lambda task: self._start_torrent_creation_process(task.result())
-            )
+        self._get_announce_url_task = self.add_task(self._get_announce_url())
+        self._get_announce_url_task.add_done_callback(
+            lambda task: self._start_torrent_creation_process(task.result())
+        )
 
     async def _get_announce_url(self):
-        self.info = 'Getting announce URL'
+        self._activity = 'announce_url'
         self.signal.emit('announce_url', Ellipsis)
         try:
             announce_url = await self._tracker.get_announce_url()
@@ -101,7 +114,7 @@ class CreateTorrentJob(base.JobBase):
             self.signal.emit('announce_url', announce_url)
             return announce_url
         finally:
-            self.info = ''
+            self._activity = ''
 
     def _start_torrent_creation_process(self, announce_url):
         self._torrent_process = daemon.DaemonProcess(
@@ -110,8 +123,10 @@ class CreateTorrentJob(base.JobBase):
             kwargs={
                 'content_path': self._content_path,
                 'torrent_path': self._torrent_path,
+                'piece_size_min': self._tracker.TrackerJobs.torrent_piece_size_min,
+                'piece_size_max': self._tracker.TrackerJobs.torrent_piece_size_max,
                 'reuse_torrent_path': self._reuse_torrent_path,
-                'overwrite': self.ignore_cache,
+                'use_cache': not self.ignore_cache,
                 'announce': announce_url,
                 'source': self._tracker.options['source'],
                 'exclude': self._exclude_files,
@@ -130,22 +145,36 @@ class CreateTorrentJob(base.JobBase):
             self._torrent_process.stop()
         super().finish()
 
+    def cancel_search(self):
+        """
+        Stop searching for a torrent we can copy hashes from
+
+        If we are not currently searching for a torrent, do nothing.
+        """
+        if self._activity == 'searching' and self._torrent_process:
+            self._torrent_process.send(daemon.MsgType.terminate, torrent.SKIP_SEARCHING)
+
     def _handle_file_tree(self, file_tree):
         self.signal.emit('file_tree', file_tree)
 
     def _handle_info_update(self, info):
         if isinstance(info, torrent.CreateTorrentProgress):
+            self._activity = 'hashing'
             self.signal.emit('progress_update', info)
-        elif isinstance(info, str):
-            # Display current filename as JobBase.info, but because it's very
-            # inefficient to update the terminal for every file name (the actual
-            # scanning process is much faster than the display), we only update
-            # in intervals.
-            now = time.monotonic()
-            prev_info_update = getattr(self, '_prev_info_update', 0)
-            if now - prev_info_update >= 0.5:
-                setattr(self, '_prev_info_update', now)
-                self.info = fs.basename(info)
+
+        elif isinstance(info, torrent.FindTorrentProgress):
+            if info.status == 'verifying':
+                self._activity = 'verifying'
+            elif info.status == 'hit':
+                _log.debug('Reusing torrent: %r', info.filepath)
+                self._activity = ''
+            else:
+                self._activity = 'searching'
+            self.signal.emit('progress_update', info)
+
+        else:
+            self._activity = ''
+            raise RuntimeError('Unexpected info update: {info!r}')
 
     def _handle_torrent_created(self, torrent_path=None):
         _log.debug('Torrent created: %r', torrent_path)
@@ -162,37 +191,41 @@ class CreateTorrentJob(base.JobBase):
 
 
 def _torrent_process(output_queue, input_queue, *args, **kwargs):
-    def terminate_signal_sent():
+    def read_input_queue():
         try:
             typ, msg = input_queue.get_nowait()
+            return typ, msg
         except queue.Empty:
-            pass
-        else:
-            if typ == daemon.MsgType.terminate:
-                return True
-        return False
+            return (None, None)
 
     def init_callback(file_tree):
-        if terminate_signal_sent():
+        typ, msg = read_input_queue()
+        if typ == daemon.MsgType.terminate:
             return 'cancel'
         else:
             output_queue.put((daemon.MsgType.init, file_tree))
 
     def progress_callback(progress):
-        if terminate_signal_sent():
-            return 'cancel'
+        typ, msg = read_input_queue()
+        if typ == daemon.MsgType.terminate:
+            if (
+                    # We were told to skip searching (not to cancel torrent creation altogether)
+                    msg == torrent.SKIP_SEARCHING
+                    # We are searching
+                    and isinstance(progress, torrent.FindTorrentProgress)
+                    # No match found yet (False = no match, None = Verifying potential match)
+                    and progress.status is not True
+            ):
+                # Stop searching for reusable torrent
+                return torrent.SKIP_SEARCHING
+            else:
+                # Cancel torrent creation altogether
+                return 'cancel'
         else:
             output_queue.put((daemon.MsgType.info, progress))
 
-    def info_callback(message):
-        if terminate_signal_sent():
-            return 'cancel'
-        else:
-            output_queue.put((daemon.MsgType.info, message))
-
     kwargs['init_callback'] = init_callback
     kwargs['progress_callback'] = progress_callback
-    kwargs['info_callback'] = info_callback
     try:
         torrent_path = torrent.create(*args, **kwargs)
     except errors.TorrentError as e:
@@ -222,21 +255,24 @@ class AddTorrentJob(base.QueueJobBase):
     label = 'Add Torrent'
     cache_id = None  # Don't cache output
 
-    def initialize(self, *, client, download_path=None, enqueue=()):
+    def initialize(self, *, client_api, download_path=None, check_after_add=False, enqueue=()):
         """
         Set internal state
 
-        :param client: Return value of :func:`.btclients.client`
+        :param client_api: :class:`aiobtclientapi.clients.APIBase` subclass
+            instance
         :param download_path: Path to the torrent's download location or `None`
             to use the client's default path
+        :param bool check_after_add: Whether to hash torrent's existing files
+            after torrent is added
         :param enqueue: Sequence of torrent file paths to add
 
         If `enqueue` is given and not empty, this job is finished as soon as its
         last item is added.
         """
-        assert isinstance(client, btclients.ClientApiBase), f'Not a ClientApiBase: {client!r}'
-        self._client = client
+        self._client_api = client_api
         self._download_path = download_path
+        self._check_after_add = check_after_add
         self.signal.add('adding')
         self.signal.add('added')
         self.signal.register('adding', lambda tp: setattr(self, 'info', f'Adding {fs.basename(tp)}'))
@@ -244,28 +280,26 @@ class AddTorrentJob(base.QueueJobBase):
         self.signal.register('finished', lambda _: setattr(self, 'info', ''))
         self.signal.register('error', lambda _: setattr(self, 'info', ''))
 
-    MAX_TORRENT_SIZE = 10 * 2**20  # 10 MiB
-    """Upper limit of acceptable size of `.torrent` files"""
-
     async def handle_input(self, torrent_path):
-        _log.debug('Adding %s to %s', torrent_path, self._client.name)
+        _log.debug('Adding %s to %s', torrent_path, self._client_api.name)
         self.signal.emit('adding', torrent_path)
 
-        if os.path.exists(torrent_path) and os.path.getsize(torrent_path) > self.MAX_TORRENT_SIZE:
-            self.error(f'{torrent_path}: File is too large')
-            return
+        response = await self._client_api.add(
+            torrent_path,
+            location=self._download_path,
+            verify=self._check_after_add,
+        )
 
-        try:
-            torrent_hash = await self._client.add_torrent(
-                torrent_path=torrent_path,
-                download_path=self._download_path,
-            )
-        except errors.RequestError as e:
-            self.error(f'Failed to add {torrent_path} to {self._client.name}: {e}')
-        else:
-            if torrent_hash:
-                self.send(torrent_hash)
-                self.signal.emit('added', torrent_hash)
+        for warning in response.warnings:
+            self.warn(warning)
+
+        for error in response.errors:
+            self.error(error)
+
+        if response.success:
+            infohash = (response.added + response.already_added)[0]
+            self.send(infohash)
+            self.signal.emit('added', infohash)
 
 
 class CopyTorrentJob(base.QueueJobBase):
