@@ -5,8 +5,10 @@ Abstract base class for tracker APIs
 import abc
 import builtins
 
-from .. import jobs
-from ..utils import (btclients, cached_property, configfiles, fs, release,
+import aiobtclientapi
+
+from .. import errors, jobs
+from ..utils import (cached_property, configfiles, fs, imghosts, release,
                      signal, string, types, webdbs)
 
 import logging  # isort:skip
@@ -33,7 +35,7 @@ class TrackerConfigBase(dict):
             value=types.Choice(
                 value='',
                 empty_ok=True,
-                options=btclients.client_names(),
+                options=aiobtclientapi.client_names(),
             ),
             description=('BitTorrent client to add torrent to after submission.'),
         ),
@@ -96,7 +98,7 @@ class TrackerJobsBase(abc.ABC):
     :attr:`jobs_after_upload` attributes. It also provides all objects that are
     needed by any one of those jobs.
 
-    Job instances are provided as :func:`~functools.cached_property`, i.e. jobs
+    Job instances are provided as :func:`functools.cached_property`, i.e. jobs
     are created only once per session.
 
     Subclasses that need to run background tasks (e.g. with
@@ -108,7 +110,7 @@ class TrackerJobsBase(abc.ABC):
     """
 
     def __init__(self, *, content_path, tracker,
-                 reuse_torrent_path=None, torrent_destination=None,
+                 reuse_torrent_path=None, torrent_destination=None, check_after_add=False,
                  exclude_files=(), options=None, image_host=None,
                  bittorrent_client=None, common_job_args=None):
         self._content_path = content_path
@@ -117,11 +119,11 @@ class TrackerJobsBase(abc.ABC):
         self._image_host = image_host
         self._bittorrent_client = bittorrent_client
         self._torrent_destination = torrent_destination
+        self._check_after_add = check_after_add
         self._exclude_files = exclude_files
         self._common_job_args = common_job_args or {}
         self._options = options or {}
         self._signal = signal.Signal('warning', 'error', 'exception')
-        self._background_tasks = []
 
     @property
     def content_path(self):
@@ -159,6 +161,11 @@ class TrackerJobsBase(abc.ABC):
         """
         return self._torrent_destination
 
+    @property
+    def check_after_add(self):
+        """Whether to hash existing files after adding the torrent to a client"""
+        return self._check_after_add
+
     @property
     def exclude_files(self):
         """
@@ -197,14 +204,17 @@ class TrackerJobsBase(abc.ABC):
         """
         return self._bittorrent_client
 
-    @property
-    def common_job_args(self):
+    def common_job_args(self, **overload):
         """
         Keyword arguments that are passed to all jobs or empty `dict`
 
-        This is the same object that was passed as a initialization argument.
+        :param overload: Keyword arguments add or replace values from the
+            initialization argument
         """
-        return self._common_job_args
+        return {
+            **self._common_job_args,
+            **overload,
+        }
 
     @property
     @abc.abstractmethod
@@ -232,27 +242,48 @@ class TrackerJobsBase(abc.ABC):
             self.copy_torrent_job,
         )
 
+    @property
+    def isolated_jobs(self):
+        """
+        Sequence of job names (e.g. ``"imdb_job"``) that were singled out by the
+        user (e.g. with a CLI argument) to create only a subset of the usual
+        metadata
+
+        If this sequence is empty, all jobs in :attr:`jobs_before_upload` and
+        :attr:`jobs_after_upload` are enabled.
+        """
+        return ()
+
     @property
     def submission_ok(self):
         """
         Whether the created metadata should be submitted
 
-        The base class implementation simply returns `True` if all
+        The base class implementation returns `False` if there are any
+        :attr:`isolated_jobs`. Otherwise, it returns `True` only if all
         :attr:`jobs_before_upload` have an :attr:`~.base.JobBase.exit_code` of
         ``0`` or a falsy :attr:`~.base.JobBase.is_enabled` value.
 
         Subclasses should always call the parent class implementation to ensure
         all metadata was created successfully.
         """
-        enabled_jobs_before_upload = tuple(
-            job for job in self.jobs_before_upload
-            if job and job.is_enabled
-        )
-        return (
-            bool(enabled_jobs_before_upload)
-            and all(job.exit_code == 0
-                    for job in enabled_jobs_before_upload)
-        )
+        if self.isolated_jobs:
+            # If some jobs are disabled, required metadata is missing and we
+            # can't submit
+            return False
+        else:
+            enabled_jobs_before_upload = tuple(
+                job for job in self.jobs_before_upload
+                if job and job.is_enabled
+            )
+            enabled_jobs_succeeded = all(
+                job.exit_code == 0
+                for job in enabled_jobs_before_upload
+            )
+            return bool(
+                enabled_jobs_before_upload
+                and enabled_jobs_succeeded
+            )
 
     @property
     def signal(self):
@@ -311,10 +342,10 @@ class TrackerJobsBase(abc.ABC):
         It's important for tracker jobs to have unique names to avoid re-using
         cached output from another tracker's job with the same name.
 
-        Standard jobs chose their names so that cached output will be re-used by
-        other trackers if possible. This function is mainly for unique and
-        custom jobs that are only used for one tracker but might share the same
-        name with other trackers.
+        Standard jobs have names so that cached output will be re-used by other
+        trackers if possible. This function is mainly for unique and custom jobs
+        that are only used for one tracker but might share the same name with
+        other trackers.
         """
         prefix = f'{self.tracker.name}-'
         if name.startswith(prefix):
@@ -330,18 +361,27 @@ class TrackerJobsBase(abc.ABC):
             reuse_torrent_path=self.reuse_torrent_path,
             tracker=self.tracker,
             exclude_files=self._exclude_files,
-            **self.common_job_args,
+            condition=self.make_job_condition('create_torrent_job'),
+            **self.common_job_args(),
         )
 
+    torrent_piece_size_min = 2**20  # 1 MiB
+    """Minimum torrent piece size for this tracker"""
+
+    torrent_piece_size_max = 16 * 2**20  # 16 MiB
+    """Maximum torrent piece size for this tracker"""
+
     @cached_property
     def add_torrent_job(self):
         """:class:`~.jobs.torrent.AddTorrentJob` instance"""
         if self.bittorrent_client and self.create_torrent_job:
             add_torrent_job = jobs.torrent.AddTorrentJob(
                 autostart=False,
-                client=self.bittorrent_client,
+                client_api=self.bittorrent_client,
                 download_path=fs.dirname(self.content_path),
-                **self.common_job_args,
+                check_after_add=self._check_after_add,
+                condition=self.make_job_condition('add_torrent_job'),
+                **self.common_job_args(),
             )
             # Pass CreateTorrentJob output to AddTorrentJob input.
             self.create_torrent_job.signal.register('output', add_torrent_job.enqueue)
@@ -359,7 +399,8 @@ class TrackerJobsBase(abc.ABC):
             copy_torrent_job = jobs.torrent.CopyTorrentJob(
                 autostart=False,
                 destination=self.torrent_destination,
-                **self.common_job_args,
+                condition=self.make_job_condition('copy_torrent_job'),
+                **self.common_job_args(),
             )
             # Pass CreateTorrentJob output to CopyTorrentJob input.
             self.create_torrent_job.signal.register('output', copy_torrent_job.enqueue)
@@ -379,8 +420,12 @@ class TrackerJobsBase(abc.ABC):
         return release.ReleaseName(
             path=self.content_path,
             translate=self.release_name_translation,
+            separator=self.release_name_separator,
         )
 
+    release_name_separator = None
+    """See :attr:`.ReleaseName.separator`"""
+
     release_name_translation = {}
     """See ``translate`` argument of :attr:`~.utils.release.ReleaseName`"""
 
@@ -392,35 +437,44 @@ class TrackerJobsBase(abc.ABC):
 
         The text is automatically updated when :attr:`imdb_job` sends an ID.
         """
-        job = jobs.dialog.TextFieldJob(
+        return jobs.dialog.TextFieldJob(
             name=self.get_job_name('release-name'),
             label='Release Name',
             text=str(self.release_name),
-            **self.common_job_args,
+            callbacks={
+                'output': self.release_name.set_release_info,
+            },
+            condition=self.make_job_condition('release_name_job'),
+            **self.common_job_args(),
         )
-        # Automatically update release_name with user-confirmed release name
-        job.signal.register('output', self.release_name.set_release_info)
-        return job
 
-    def _update_release_name(self, imdb_id):
-        self.release_name_job.add_task(
-            self.release_name_job.fetch_text(
-                coro=self.release_name.fetch_info(imdb_id=imdb_id),
-                error_is_fatal=False,
-            ),
+    async def _update_release_name_from(self, webdb, webdb_id):
+        await self.release_name_job.fetch_text(
+            coro=self.release_name.fetch_info(webdb=webdb, webdb_id=webdb_id),
+            nonfatal_exceptions=(errors.RequestError,),
         )
+        _log.debug('Updated release name: %s', self.release_name)
 
     @cached_property
     def imdb_job(self):
         """:class:`~.jobs.webdb.WebDbSearchJob` instance"""
-        imdb_job = jobs.webdb.WebDbSearchJob(
+        return jobs.webdb.WebDbSearchJob(
             query=self.content_path,
             db=self.imdb,
-            **self.common_job_args,
+            callbacks={
+                'output': self._handle_imdb_id,
+            },
+            condition=self.make_job_condition('imdb_job'),
+            **self.common_job_args(),
         )
-        # Update release name with IMDb data
-        imdb_job.signal.register('output', self._update_release_name)
-        return imdb_job
+
+    def _handle_imdb_id(self, imdb_id):
+        # Update other webdb queries with IMDb info
+        self.release_name_job.add_task(self._propagate_webdb_info(self.imdb, imdb_id))
+
+        # Upload poster image from appropriate webdb
+        if self._poster_webdb is self.imdb:
+            self._start_poster_job(self.imdb, imdb_id)
 
     @cached_property
     def tmdb_job(self):
@@ -428,18 +482,62 @@ class TrackerJobsBase(abc.ABC):
         return jobs.webdb.WebDbSearchJob(
             query=self.content_path,
             db=self.tmdb,
-            **self.common_job_args,
+            callbacks={
+                'output': self._handle_tmdb_id,
+            },
+            condition=self.make_job_condition('tmdb_job'),
+            **self.common_job_args(),
         )
 
+    def _handle_tmdb_id(self, tmdb_id):
+        # Update other webdb queries with TMDb info
+        self.release_name_job.add_task(self._propagate_webdb_info(self.tmdb, tmdb_id))
+
+        if self._poster_webdb is self.tmdb:
+            self._start_poster_job(self.tmdb, tmdb_id)
+
     @cached_property
     def tvmaze_job(self):
         """:class:`~.jobs.webdb.WebDbSearchJob` instance"""
         return jobs.webdb.WebDbSearchJob(
             query=self.content_path,
             db=self.tvmaze,
-            **self.common_job_args,
+            callbacks={
+                'output': self._handle_tvmaze_id,
+            },
+            condition=self.make_job_condition('tvmaze_job'),
+            **self.common_job_args(),
         )
 
+    def _handle_tvmaze_id(self, tvmaze_id):
+        # Update other webdb queries with TVmaze info
+        self.release_name_job.add_task(self._propagate_webdb_info(self.tvmaze, tvmaze_id))
+
+        if self._poster_webdb is self.tvmaze:
+            self._start_poster_job(self.tvmaze, tvmaze_id)
+
+    async def _propagate_webdb_info(self, webdb, webdb_id):
+        target_webdb_jobs = [
+            j for j in (getattr(self, f'{name}_job') for name in webdbs.webdb_names())
+            if (
+                webdb.name not in j.name
+                and j.is_enabled
+                and not j.is_finished
+            )
+        ]
+
+        query = webdbs.Query(
+            type=await webdb.type(webdb_id),
+            title=await webdb.title_original(webdb_id),
+            year=await webdb.year(webdb_id),
+        )
+
+        _log.debug('Propagating %s info to: %r: %s', webdb.name, [j.name for j in target_webdb_jobs], query)
+        for job in target_webdb_jobs:
+            job.query.update(query)
+
+        await self._update_release_name_from(webdb, webdb_id)
+
     @cached_property
     def screenshots_job(self):
         """
@@ -451,7 +549,8 @@ class TrackerJobsBase(abc.ABC):
         return jobs.screenshots.ScreenshotsJob(
             content_path=self.content_path,
             count=self.options.get('screenshots'),
-            **self.common_job_args,
+            condition=self.make_job_condition('screenshots_job'),
+            **self.common_job_args(),
         )
 
     image_host_config = {}
@@ -477,7 +576,8 @@ class TrackerJobsBase(abc.ABC):
         if self.image_host and self.screenshots_job:
             imghost_job = jobs.imghost.ImageHostJob(
                 imghost=self.image_host,
-                **self.common_job_args,
+                condition=self.make_job_condition('upload_screenshots_job'),
+                **self.common_job_args(),
             )
             # Timestamps are calculated in a subprocess, we have to wait for
             # that until we can set the number of expected screenhots.
@@ -494,89 +594,126 @@ class TrackerJobsBase(abc.ABC):
     def finalize_upload_screenshots_job(self, _):
         self.upload_screenshots_job.finalize()
 
+    @cached_property
+    def poster_job(self):
+        """
+        :class:`~.jobs.poster.PosterJob` instance
+
+        This is started automatically when an appropriate webdb job finishes.
+        """
+        return jobs.poster.PosterJob(
+            autostart=False,
+            webdb_id=None,
+            webdb=None,
+            imghost=self.image_host,
+            width=self.poster_max_width,
+            height=self.poster_max_height,
+            condition=self.make_job_condition('poster_job'),
+            **self.common_job_args(),
+        )
+
+    def _start_poster_job(self, webdb, webdb_id):
+        self.poster_job.start(
+            webdb=webdb,
+            webdb_id=webdb_id,
+            season=self.release_name.only_season,
+        )
+
+    poster_max_width = 300
+    """Maximum poster image width"""
+
+    poster_max_height = 500
+    """Maximum poster image height"""
+
+    @cached_property
+    def _poster_webdb(self):
+        """:class:`~.webdbs.base.WebDbApiBase` instance for finding poster image"""
+        if (
+                self.poster_job.is_enabled
+                and not self.poster_job.is_started
+                and self.poster_job in self.jobs_before_upload
+        ):
+            if (
+                    self.tvmaze_job.is_enabled
+                    and self.tvmaze_job in self.jobs_before_upload
+                    and self.release_name.type in (release.ReleaseType.season,
+                                                   release.ReleaseType.episode)
+            ):
+                return self.tvmaze
+
+            elif (
+                    self.imdb_job.is_enabled
+                    and self.imdb_job in self.jobs_before_upload
+            ):
+                return self.imdb
+
+            elif (
+                    self.tmdb_job.is_enabled
+                    and self.tmdb_job in self.jobs_before_upload
+            ):
+                return self.tmdb
+
+            else:
+                raise RuntimeError('You must add a WebDbSearchJob instance to jobs_before_upload')
+
     @cached_property
     def mediainfo_job(self):
         """:class:`~.jobs.mediainfo.MediainfoJob` instance"""
         return jobs.mediainfo.MediainfoJob(
             content_path=self.content_path,
-            **self.common_job_args,
+            condition=self.make_job_condition('mediainfo_job'),
+            **self.common_job_args(),
         )
 
     @cached_property
     def scene_check_job(self):
         """:class:`~.jobs.scene.SceneCheckJob` instance"""
-        kwargs = self.common_job_args.copy()
-        if self.options.get('is_scene') is not None:
-            kwargs['force'] = self.options.get('is_scene')
-            # Don't use cached value if the user made a choice
-            kwargs['ignore_cache'] = True
+        common_job_args = self.common_job_args(ignore_cache=True)
+        common_job_args['force'] = self.options.get('is_scene')
         return jobs.scene.SceneCheckJob(
             content_path=self.content_path,
-            **kwargs,
+            condition=self.make_job_condition('scene_check_job'),
+            **common_job_args,
         )
 
-    def make_choice_job(self, name, label, options, condition=None,
-                        callbacks=None, autodetected=None, autofinish=False):
+    def make_job_condition(self, job_attr):
         """
-        Return :class:`~.jobs.dialog.ChoiceJob` instance
+        Return :attr:`~.base.JobBase.condition` function for job
 
-        :param name: See :class:`~.jobs.dialog.ChoiceJob`
-        :param label: See :class:`~.jobs.dialog.ChoiceJob`
-        :param autodetected: Autodetected choice
+        :param str job_attr: Name of the job attribute this condition is for
 
-        :param options: Sequence of :class:`dict`-like objects with the
-            following keys:
+            By convention, this should be ``"<name>_job"``.
 
-            - ``label``: User-facing value
-
-            - ``value``: Internal value that is made available via the
-              :attr:`~.dialog.ChoiceJob.choice` attribute of the returned object
-
-            - ``match`` (Optional): Callable that gets `autodetected`; if its return value
-              is truthy, this option is autofocused
-
-            - ``regex`` (Optional): Regular expression; if this matches `autodetected`,
-              this option is autofocused
-
-        :param condition: See :attr:`~.base.JobBase.condition`
-        :param callbacks: See :attr:`~.base.JobBase`
-        :param bool autofinish: Whether to choose the autodetected value with no
-            user-interaction
+        The returned function takes into account :attr:`jobs_before_upload`,
+        :attr:`jobs_after_upload` and :attr:`isolated_job`.
         """
-        def is_autodetected(option):
-            if autodetected:
-                regex = option.get('regex')
-                match = option.get('match')
-                if regex and regex.search(str(autodetected)):
-                    return True
-                elif match and match(autodetected):
-                    return True
+        def condition():
+            job = getattr(self, job_attr)
+            if not (
+                    job in self.jobs_before_upload
+                    or job in self.jobs_after_upload
+            ):
+                # Subclass doesn't use this job
+                return False
+
+            isolated_jobs = self.isolated_jobs
+            if isolated_jobs and job in isolated_jobs:
+                # Jobs was isolated by user (i.e. other jobs are disabled)
+                return True
+
+            if not isolated_jobs:
+                # No isolated jobs means all jobs in jobs_before/after_upload are enabled
+                return True
+
             return False
 
-        focused = None
-        choices = []
-        for option in options:
-            if not focused and is_autodetected(option):
-                choices.append((f'{option["label"]} (autodetected)', option['value']))
-                focused = choices[-1]
-                autofinish = autofinish and True
-            else:
-                choices.append((option['label'], option['value']))
+        # Rename condition function to make debugging more readable
+        condition.__qualname__ = f'{job_attr}_condition'
+        return condition
 
-        job = jobs.dialog.ChoiceJob(
-            name=self.get_job_name(name),
-            label=label,
-            condition=condition,
-            callbacks=callbacks or {},
-            choices=choices,
-            focused=focused,
-            **self.common_job_args,
-        )
-        if autofinish and focused:
-            job.choice = focused
-        return job
+    _NO_DEFAULT = object()
 
-    def get_job_output(self, job, slice=None):
+    def get_job_output(self, job, slice=None, default=_NO_DEFAULT):
         """
         Helper method for getting output from job
 
@@ -585,31 +722,42 @@ class TrackerJobsBase(abc.ABC):
         :param job: :class:`~.jobs.base.JobBase` instance
         :param slice: :class:`int` to get a specific item from `job`'s output,
             `None` to return all output as a list or a :class:`slice` object
+        :param default: Default value if `job` is not finished or getting
+            `slice` from `job`'s output fails.
 
         :raise RuntimeError: if `job` is not finished or getting `slice` from
             :attr:`~.base.JobBase.output` raises an :class:`IndexError`
         :return: :class:`list` or :class:`str`
         """
         if not job.is_finished:
-            raise RuntimeError(f'Unfinished job: {job.name}')
-        if slice is None:
-            slice = builtins.slice(None, None)
-        try:
-            return job.output[slice]
-        except IndexError:
-            raise RuntimeError(f'Job finished with insufficient output: {job.name}: {job.output}')
+            if default is not self._NO_DEFAULT:
+                return default
+            else:
+                raise RuntimeError(f'Unfinished job: {job.name}')
+        else:
+            if slice is None:
+                slice = builtins.slice(None, None)
+            try:
+                return job.output[slice]
+            except IndexError:
+                raise RuntimeError(f'Job finished with insufficient output: {job.name}: {job.output}')
 
-    def get_job_attribute(self, job, attribute):
+    def get_job_attribute(self, job, attribute, default=_NO_DEFAULT):
         """
         Helper method for getting an attribute from job
 
-        `job` must be finished.
+        :param job: :class:`~.jobs.base.JobBase` instance
+        :param str attribute: Name of attribute to get from `job`
+        :param default: Default value if `job` is not finished
 
         :raise RuntimeError: if `job` is not finished
         :raise AttributeError: if `attribute` is not an attribute of `job`
         """
         if not job.is_finished:
-            raise RuntimeError(f'Unfinished job: {job.name}')
+            if default is not self._NO_DEFAULT:
+                return default
+            else:
+                raise RuntimeError(f'Unfinished job: {job.name}')
         else:
             return getattr(job, attribute)
 
@@ -763,6 +911,25 @@ class _Howto:
             ),
         ))
 
+    @property
+    def screenshots(self):
+        return self._autobump((
+            (
+                f'{self._section}. Screenshots (Optional)\n'
+                '\n'
+                f'   {self._section}.1 Specify how many screenshots to make.\n'
+                f'       $ upsies set trackers.{self._tracker_cls.name}.screenshots NUMBER_OF_SCREENSHOTS\n'
+            ),
+            (
+                f'   {self._section}.2 Specify where to host images.\n'
+                f'       $ upsies set trackers.{self._tracker_cls.name}.image_host IMAGE_HOST\n'
+                f'       Supported services: ' + ', '.join(imghosts.imghost_names()) + '\n'
+                '\n'
+                f'   {self._section}.3 Configure image hosting service.\n'
+                f'       $ upsies upload-images IMAGE_HOST --help\n'
+            ),
+        ))
+
     @property
     def autoseed(self):
         return self._autobump((
@@ -770,8 +937,8 @@ class _Howto:
                 f'{self._section}. Add Uploaded Torrents To Client (Optional)\n'
                 '\n'
                 f'   {self._section}.1 Specify which client to add uploaded torrents to.\n'
-                f'       $ upsies set trackers.{self._tracker_cls.name}.add-to CLIENT_NAME\n'
-                f'       Supported clients: ' + ', '.join(btclients.client_names()) + '\n'
+                f'       $ upsies set trackers.{self._tracker_cls.name}.add_to CLIENT_NAME\n'
+                f'       Supported clients: ' + ', '.join(aiobtclientapi.client_names()) + '\n'
             ),
             (
                 f'   {self._section}.2 Specify your client connection.\n'
@@ -781,7 +948,7 @@ class _Howto:
                 '\n'
                 f'{self._section + 1}. Copy Uploaded Torrents To Directory (Optional)\n'
                 '\n'
-                f'   $ upsies set trackers.{self._tracker_cls.name}.copy-to /path/to/directory\n'
+                f'   $ upsies set trackers.{self._tracker_cls.name}.copy_to /path/to/directory\n'
             ),
         ))
 
