@@ -4,7 +4,6 @@ Concrete :class:`~.base.TrackerJobsBase` subclass for BHD
 
 import io
 import os
-import re
 
 from ... import __homepage__, __project_name__, jobs
 from ...utils import as_groups, cached_property, release, string
@@ -15,28 +14,9 @@ _log = logging.getLogger(__name__)
 
 
 class BhdTrackerJobs(TrackerJobsBase):
-    release_name_translation = {
-        'audio_format': {
-            # Don't match against end of string to preserve extensions like
-            # "Atmos"
-            re.compile(r'^AC-3'): r'DD',
-            re.compile(r'^E-AC-3'): r'DDP',
-        },
-    }
 
     @cached_property
     def jobs_before_upload(self):
-        # Make base class jobs conditional. Custom BHD jobs get their condition
-        # in their properties.
-        generic_job_attributes = ('imdb_job', 'tmdb_job', 'release_name_job',
-                                  'scene_check_job', 'create_torrent_job', 'mediainfo_job',
-                                  'screenshots_job', 'upload_screenshots_job',
-                                  'add_torrent_job', 'copy_torrent_job')
-        for job_attr in generic_job_attributes:
-            job = getattr(self, job_attr, None)
-            if job:
-                job.condition = self.make_job_condition(job_attr)
-
         return (
             # Background jobs
             self.create_torrent_job,
@@ -45,8 +25,8 @@ class BhdTrackerJobs(TrackerJobsBase):
             self.upload_screenshots_job,
 
             # Interactive jobs
-            self.imdb_job,
             self.tmdb_job,
+            self.imdb_job,
             self.release_name_job,
             self.category_job,
             self.type_job,
@@ -56,21 +36,6 @@ class BhdTrackerJobs(TrackerJobsBase):
             self.tags_job,
         )
 
-    def make_job_condition(self, job_attr):
-        """
-        Return :attr:`~.base.JobBase.condition` for jobs
-
-        :param str job_attr: Name of the job attribute this condition is for
-        """
-        def condition():
-            isolated_jobs = self.isolated_jobs
-            if isolated_jobs:
-                return job_attr in isolated_jobs
-            else:
-                return True
-
-        return condition
-
     @property
     def isolated_jobs(self):
         """
@@ -79,36 +44,29 @@ class BhdTrackerJobs(TrackerJobsBase):
         """
         if self.options.get('only_description', False):
             # Only activate description_job and its dependencies
-            return ('description_job', 'screenshots_job', 'upload_screenshots_job')
+            return (self.description_job, self.screenshots_job, self.upload_screenshots_job)
         elif self.options.get('only_title', False):
             # Only activate release_name_job and its dependencies
-            return ('release_name_job', 'imdb_job')
+            return (self.release_name_job, self.imdb_job)
         else:
+            # Activate all jobs in jobs_before/after_upload
             return ()
 
-    @cached_property
-    def imdb_job(self):
-        imdb_job = super().imdb_job
-        imdb_job.signal.register('output', self.handle_imdb_job_output)
-        return imdb_job
-
-    def handle_imdb_job_output(self, _):
-        user_confirmed_type = self.imdb_job.query.type
-        self.tmdb_job.query.type = user_confirmed_type
-
     @cached_property
     def category_job(self):
-        # 'output' signal is only emitted when job succeeds while 'finished'
-        # signal is also emitted when job fails (e.g. when Ctrl-c is pressed)
-        self.release_name_job.signal.register('output', self.autodetect_category)
-        return self.make_choice_job(
+        return jobs.dialog.ChoiceJob(
             name=self.get_job_name('category'),
             label='Category',
             condition=self.make_job_condition('category_job'),
-            options=(
-                {'label': 'Movie', 'value': '1'},
-                {'label': 'TV', 'value': '2'},
+            prejobs=(
+                self.release_name_job,
             ),
+            autodetect=self.autodetect_category,
+            choices=(
+                ('Movie', '1'),
+                ('TV', '2'),
+            ),
+            **self.common_job_args(),
         )
 
     _autodetect_category_map = {
@@ -122,43 +80,40 @@ class BhdTrackerJobs(TrackerJobsBase):
         _log.debug('Autodetecting category: Approved release type: %r', approved_release_name.type)
         for label, is_match in self._autodetect_category_map.items():
             if is_match(approved_release_name):
-                # Focus autodetected choice
-                self.category_job.focused = label
-                # Get value of autodetected choice
-                value = self.category_job.focused[1]
-                # Mark autodetected choice
-                self.category_job.set_label(value, f'{label} (autodetected)')
-                break
+                return label
 
     @cached_property
     def type_job(self):
-        # 'output' signal is only emitted when job succeeds while 'finished'
-        # signal is also emitted when job fails (e.g. when Ctrl-c is pressed)
-        self.release_name_job.signal.register('output', self.autodetect_type)
-        return self.make_choice_job(
+        return jobs.dialog.ChoiceJob(
             name=self.get_job_name('type'),
             label='Type',
             condition=self.make_job_condition('type_job'),
-            options=(
-                {'label': 'UHD 100', 'value': 'UHD 100'},
-                {'label': 'UHD 66', 'value': 'UHD 66'},
-                {'label': 'UHD 50', 'value': 'UHD 50'},
-                {'label': 'UHD Remux', 'value': 'UHD Remux'},
-                {'label': 'BD 50', 'value': 'BD 50'},
-                {'label': 'BD 25', 'value': 'BD 25'},
-                {'label': 'BD Remux', 'value': 'BD Remux'},
-                {'label': '2160p', 'value': '2160p'},
-                {'label': '1080p', 'value': '1080p'},
-                {'label': '1080i', 'value': '1080i'},
-                {'label': '720p', 'value': '720p'},
-                {'label': '576p', 'value': '576p'},
-                {'label': '540p', 'value': '540p'},
-                {'label': 'DVD 9', 'value': 'DVD 9'},
-                {'label': 'DVD 5', 'value': 'DVD 5'},
-                {'label': 'DVD Remux', 'value': 'DVD Remux'},
-                {'label': '480p', 'value': '480p'},
-                {'label': 'Other', 'value': 'Other'},
+            prejobs=(
+                self.release_name_job,
             ),
+            autodetect_and_finish=self.autodetect_type,
+            choices=(
+                ('UHD 100', 'UHD 100'),
+                ('UHD 66', 'UHD 66'),
+                ('UHD 50', 'UHD 50'),
+                ('UHD Remux', 'UHD Remux'),
+                ('BD 50', 'BD 50'),
+                ('BD 25', 'BD 25'),
+                ('BD Remux', 'BD Remux'),
+                ('2160p', '2160p'),
+                ('1080p', '1080p'),
+                ('1080i', '1080i'),
+                ('720p', '720p'),
+                ('576p', '576p'),
+                ('540p', '540p'),
+                ('DVD 9', 'DVD 9'),
+                ('DVD 5', 'DVD 5'),
+                ('DVD Remux', 'DVD Remux'),
+                ('480p', '480p'),
+                ('Other', 'Other'),
+            ),
+            focused='Other',
+            **self.common_job_args(),
         )
 
     _autodetect_type_map = {
@@ -180,34 +135,30 @@ class BhdTrackerJobs(TrackerJobsBase):
         _log.debug('Autodetecting type: Approved source: %r', approved_release_name.source)
         for label, is_match in self._autodetect_type_map.items():
             if is_match(approved_release_name):
-                # Focus autodetected choice
-                self.type_job.focused = label
-                # Get value of autodetected choice
-                value = self.type_job.focused[1]
-                # Mark autodetected choice
-                self.type_job.set_label(value, f'{label} (autodetected)')
-                # Select autodetected choice (i.e. finish job and don't prompt user)
-                self.type_job.choice = value
-                break
-        else:
-            self.type_job.focused = 'Other'
+                return label
 
     @cached_property
     def source_job(self):
         # 'output' signal is only emitted when job succeeds while 'finished'
         # signal is also emitted when job fails (e.g. when Ctrl-c is pressed)
         self.release_name_job.signal.register('output', self.autodetect_source)
-        return self.make_choice_job(
+
+        return jobs.dialog.ChoiceJob(
             name=self.get_job_name('source'),
             label='Source',
             condition=self.make_job_condition('source_job'),
-            options=(
-                {'label': 'Blu-ray', 'value': 'Blu-ray'},
-                {'label': 'HD-DVD', 'value': 'HD-DVD'},
-                {'label': 'WEB', 'value': 'WEB'},
-                {'label': 'HDTV', 'value': 'HDTV'},
-                {'label': 'DVD', 'value': 'DVD'},
+            prejobs=(
+                self.release_name_job,
             ),
+            autodetect_and_finish=self.autodetect_source,
+            choices=(
+                ('Blu-ray', 'Blu-ray'),
+                ('HD-DVD', 'HD-DVD'),
+                ('WEB', 'WEB'),
+                ('HDTV', 'HDTV'),
+                ('DVD', 'DVD'),
+            ),
+            **self.common_job_args(),
         )
 
     # Map type_job labels to matchers
@@ -224,15 +175,7 @@ class BhdTrackerJobs(TrackerJobsBase):
         _log.debug('Autodetecting source: Approved source: %r', approved_release_name.source)
         for label, is_match in self._autodetect_source_map.items():
             if is_match(approved_release_name):
-                # Focus autodetected choice
-                self.source_job.focused = label
-                # Get value of autodetected choice
-                value = self.source_job.focused[1]
-                # Mark autodetected choice
-                self.source_job.set_label(value, f'{label} (autodetected)')
-                # Select autodetected choice (i.e. finish job and don't prompt user)
-                self.source_job.choice = value
-                break
+                return label
 
     @cached_property
     def description_job(self):
@@ -241,7 +184,7 @@ class BhdTrackerJobs(TrackerJobsBase):
             label='Description',
             condition=self.make_job_condition('description_job'),
             read_only=True,
-            **self.common_job_args,
+            **self.common_job_args(ignore_cache=True),
         )
         job.add_task(
             job.fetch_text(
@@ -288,7 +231,7 @@ class BhdTrackerJobs(TrackerJobsBase):
             label='Tags',
             condition=self.make_job_condition('tags_job'),
             read_only=True,
-            **self.common_job_args,
+            **self.common_job_args(),
         )
         job.add_task(
             job.fetch_text(
@@ -335,17 +278,6 @@ class BhdTrackerJobs(TrackerJobsBase):
 
         return '\n'.join(tags)
 
-    @property
-    def submission_ok(self):
-        """
-        `False` if :attr:`~.TrackerJobsBase.options` prevents submission for any
-        reason, parent class implementation otherwise
-        """
-        if self.isolated_jobs:
-            return False
-        else:
-            return super().submission_ok
-
     @property
     def post_data(self):
         return {
