@@ -2,7 +2,7 @@
 Generate properly formatted release name
 """
 
-from .... import jobs, utils
+from .... import errors, jobs, utils
 from .base import CommandBase
 
 
@@ -26,6 +26,10 @@ class release_name(CommandBase):
             'type': utils.argtypes.release,
             'help': 'Release name or path to release content',
         },
+        ('--separator', '-s'): {
+            'default': ' ',
+            'help': 'Separator between parts (default: " ")',
+        },
     }
 
     @utils.cached_property
@@ -34,7 +38,10 @@ class release_name(CommandBase):
 
     @utils.cached_property
     def release_name(self):
-        return utils.release.ReleaseName(self.args.RELEASE)
+        return utils.release.ReleaseName(
+            self.args.RELEASE,
+            separator=self.args.separator,
+        )
 
     @utils.cached_property
     def release_name_job(self):
@@ -50,8 +57,10 @@ class release_name(CommandBase):
     def _update_release_name(self, imdb_id):
         self.release_name_job.add_task(
             self.release_name_job.fetch_text(
-                coro=self.release_name.fetch_info(imdb_id=imdb_id),
-                error_is_fatal=False,
+                coro=self.release_name.fetch_info(webdb=self.imdb, webdb_id=imdb_id),
+                nonfatal_exceptions=(
+                    errors.RequestError,
+                ),
             ),
         )
 
@@ -63,7 +72,11 @@ class release_name(CommandBase):
             home_directory=self.home_directory,
             cache_directory=self.cache_directory,
             ignore_cache=self.args.ignore_cache,
-            query=self.args.RELEASE,
-            db=utils.webdbs.webdb('imdb'),
+            query=utils.webdbs.Query.from_path(self.args.RELEASE),
+            db=self.imdb,
             callbacks={'output': self._update_release_name},
         )
+
+    @utils.cached_property
+    def imdb(self):
+        return utils.webdbs.webdb('imdb')
