@@ -72,7 +72,6 @@ class TUI:
             full_screen=False,
             erase_when_done=False,
             mouse_support=False,
-            on_invalidate=self._update_jobs_container,
         )
         # Make escape key work
         app.timeoutlen = 0.1
@@ -81,36 +80,52 @@ class TUI:
 
     def add_jobs(self, *jobs):
         """Add :class:`~.jobs.base.JobBase` instances"""
-        for job in jobs:
-            if job.name in self._jobs:
-                raise RuntimeError(f'Job was already added: {job.name}')
-            else:
-                self._jobs[job.name].job = job
-                self._jobs[job.name].widget = jobwidgets.JobWidget(job, self._app)
-                self._jobs[job.name].container = to_container(self._jobs[job.name].widget)
 
-                # Terminate application if all jobs finished
-                job.signal.register('finished', self._exit_if_all_jobs_finished)
-
-                # Terminate application if any job finished with non-zero exit code
-                job.signal.register('finished', self._exit_if_job_failed)
-
-                if self._jobs[job.name].widget.is_interactive:
-                    # Display next interactive job when currently focused job finishes
-                    job.signal.register('finished', self._update_jobs_container)
+        def recurse(*jobs):
+            for job in jobs:
+                for prejob in job.prejobs:
+                    recurse(prejob)
+                self._add_job(job)
 
+        recurse(*jobs)
         self._update_jobs_container()
 
+    def _add_job(self, job):
+        if job.name in self._jobs:
+            if job is not self._jobs[job.name].job:
+                raise RuntimeError(f'Conflicting job name: {job.name}')
+        else:
+            self._jobs[job.name].job = job
+            self._jobs[job.name].widget = jobwidgets.JobWidget(job, self._app)
+            self._jobs[job.name].container = to_container(self._jobs[job.name].widget)
+
+            # Terminate application if all jobs finished
+            job.signal.register('finished', self._exit_if_all_jobs_finished)
+
+            # Terminate application if any job finished with non-zero exit code
+            job.signal.register('finished', self._exit_if_job_failed)
+
+            if self._jobs[job.name].widget.is_interactive:
+                # Display next interactive job when this interactive job finishes
+                job.signal.register('finished', self._update_jobs_container)
+
     def _update_jobs_container(self, *_):
         job_containers = []
+        enabled_jobs = self._enabled_jobs
 
         # Start all enabled jobs that are supposed to start automatically
-        for jobinfo in self._enabled_jobs:
+        for jobinfo in enabled_jobs:
             if not jobinfo.job.is_started and jobinfo.job.autostart:
                 jobinfo.job.start()
 
+        # Unfocus currently focused widget
+        for jobinfo in enabled_jobs:
+            if jobinfo.widget.is_focused:
+                jobinfo.widget.focus_lost()
+                break
+
         # List interactive jobs first
-        for jobinfo in self._enabled_jobs:
+        for jobinfo in enabled_jobs:
             if jobinfo.widget.is_interactive:
                 job_containers.append(jobinfo.container)
 
@@ -120,9 +135,9 @@ class TUI:
                         self._layout.focus(jobinfo.container)
                     except ValueError:
                         pass
-                    #     _log.debug('Failed to focus job: %r', jobinfo.job.name)
-                    # else:
-                    #     _log.debug('Focused job: %r', jobinfo.job.name)
+                        # _log.debug('Unfocusable job: %r', jobinfo.job.name)
+                    else:
+                        jobinfo.widget.focus_gained()
 
                     # Don't display more than one unfinished interactive job
                     # unless any job has errors, in which case we are
@@ -133,7 +148,7 @@ class TUI:
         # Add non-interactive jobs below interactive jobs so the interactive
         # widgets don't change position when non-interactive widgets change
         # size.
-        for jobinfo in self._enabled_jobs:
+        for jobinfo in enabled_jobs:
             if not jobinfo.widget.is_interactive:
                 job_containers.append(jobinfo.container)
 
