@@ -3,6 +3,7 @@ Custom TUI widgets
 """
 
 import collections
+import itertools
 import textwrap
 
 from prompt_toolkit.application import get_app
@@ -15,9 +16,8 @@ from prompt_toolkit.layout.containers import (HSplit, VSplit, Window,
                                               WindowAlign)
 from prompt_toolkit.layout.controls import BufferControl, FormattedTextControl
 from prompt_toolkit.layout.dimension import Dimension
-from prompt_toolkit.utils import get_cwidth
 
-from . import utils
+from ...utils import get_aioloop
 
 import logging  # isort:skip
 _log = logging.getLogger(__name__)
@@ -34,7 +34,7 @@ class TextField:
         self._text = text
         self._width = width
         self._height = height
-        self._activity_indicator = utils.ActivityIndicator(callback=self.set_text)
+        self._activity_indicator = ActivityIndicator(callback=self.set_text)
         self.container = Window(
             content=FormattedTextControl(lambda: self.text),
             width=width,
@@ -98,7 +98,7 @@ class InputField:
 
     def __init__(self, text='', width=None, extend_width=True, read_only=False,
                  on_accepted=None, on_changed=None, style=''):
-        self._activity_indicator = utils.ActivityIndicator(callback=self.set_text)
+        self._activity_indicator = ActivityIndicator(callback=self.set_text)
         self.read_only = read_only
         self.on_accepted = on_accepted
         self.buffer = Buffer(
@@ -178,14 +178,20 @@ class RadioList:
     complete choice item.
     """
 
-    def __init__(self, choices=(), focused=None, on_accepted=None):
+    def __init__(self, choices=(), focused=None, autodetected=None, on_accepted=None):
         self.choices = choices
         self.on_accepted = on_accepted
+
         if focused:
             self.focused_index = choices.index(focused)
         else:
             self.focused_index = 0
 
+        if autodetected:
+            self.autodetected_index = choices.index(autodetected)
+        else:
+            self.autodetected_index = None
+
         kb = KeyBindings()
 
         @kb.add('up')
@@ -253,12 +259,14 @@ class RadioList:
             if i == self.focused_index:
                 style = 'class:dialog.choice.focused'
                 fragments.append(('[SetCursorPosition]', ''))
-                fragments.append((style, '*'))
+                fragments.append((style, ' *'))
             else:
                 style = 'class:dialog.choice'
-                fragments.append((style, ' '))
+                fragments.append((style, '  '))
             fragments.append((style, ' '))
-            fragments.extend(to_formatted_text(choice_string.ljust(width), style=style))
+            fragments.extend(to_formatted_text(choice_string.ljust(width) + ' ', style=style))
+            if i == self.autodetected_index:
+                fragments.append(('class:dialog.choice', '(autodetected)'))
             fragments.append(('', '\n'))
 
         fragments.pop()  # Remove last newline
@@ -273,10 +281,8 @@ class HLabel:
     _groups = collections.defaultdict(lambda: [])
 
     def __init__(self, text, content, group=None, style=''):
-        self.text = text
-        self.width = get_cwidth(text)
         self.label = Window(
-            content=FormattedTextControl(text=text),
+            content=FormattedTextControl(text),
             dont_extend_width=True,
             dont_extend_height=True,
             style=style,
@@ -294,7 +300,8 @@ class HLabel:
         self._group = group
         if group is not None:
             self._groups[group].append(self)
-            max_width = max(label.width for label in self._groups[group])
+            texts = [label.label.content.text for label in self._groups[group]]
+            max_width = max(len(text) for text in texts)
             for label in self._groups[group]:
                 label.label.width = max_width + 1
 
@@ -347,3 +354,90 @@ class ProgressBar:
 
     def __pt_container__(self):
         return self.container
+
+
+class ActivityIndicator:
+    """
+    Activity indicator that cycles through a sequence of strings
+
+    :param callable callback: Callable that receives a single string from
+        `states` as a positional argument
+    :param states: Sequence of strings that are cycled through and passed to
+        `callback` individually
+    :param float interval: Delay between calls to `callback`
+    :param bool active: Whether `callback` is called right away
+    :param str format: See :attr:`format`
+    :param str style: prompt-toolkit style
+    :param bool extend_width: Whether to maximize horizontally
+    """
+
+    def __init__(self, *, callback=None, states=('⠷', '⠯', '⠟', '⠻', '⠽', '⠾'),
+                 interval=0.3, active=False, format='{indicator}', style='',
+                 extend_width=False):
+        self._iterator = itertools.cycle(states)
+        self._interval = float(interval)
+        self._callback = callback or None
+        self._format = format
+        self._text = self._format.format(indicator=states[0])
+        self.active = active
+
+        self.container = Window(
+            content=FormattedTextControl(lambda: self.text),
+            height=1,
+            width=Dimension(min=len(self.text)),
+            dont_extend_height=True,
+            dont_extend_width=not extend_width,
+            style=style,
+        )
+
+    @property
+    def active(self):
+        """Whether `callback` is being called continuously"""
+        return getattr(self, '_active', False)
+
+    @active.setter
+    def active(self, active):
+        if active:
+            if not getattr(self, '_active', None):
+                self._active = True
+                self._iterate()
+        else:
+            self._active = False
+
+    def enable(self):
+        """Calling this method sets :attr:`active` to `True`"""
+        self.active = True
+
+    def disable(self):
+        """Calling this method sets :attr:`active` to `False`"""
+        self.active = False
+
+    @property
+    def format(self):
+        """
+        Format string
+
+        ``{indicator}`` is replaced with the current state in `states`.
+        """
+        return self._format
+
+    @format.setter
+    def format(self, format):
+        self._format = format
+
+    @property
+    def text(self):
+        """Formatted :attr:`format` string"""
+        return self._text
+
+    def _iterate(self):
+        if self.active:
+            self._text = self._format.format(indicator=next(self._iterator))
+            if self._callback:
+                self._callback(self._text)
+
+            get_aioloop().call_later(self._interval, self._iterate)
+            get_app().invalidate()
+
+    def __pt_container__(self):
+        return self.container
