@@ -21,10 +21,7 @@ def assert_file_readable(filepath):
     try:
         open(filepath).close()
     except OSError as e:
-        if e.strerror:
-            raise errors.ContentError(f'{filepath}: {e.strerror}')
-        else:
-            raise errors.ContentError(f'{filepath}: {e}')
+        raise errors.ContentError(f'{filepath}: {e.strerror or e}')
 
 
 def assert_dir_usable(path):
@@ -127,10 +124,7 @@ def prune_empty(path, files=False, directories=True):
         return
 
     def raise_error(e, path):
-        if e.strerror:
-            raise RuntimeError(f'{path}: Failed to prune: {e.strerror}')
-        else:
-            raise RuntimeError(f'{path}: Failed to prune: {e}')
+        raise RuntimeError(f'{path}: Failed to prune: {e.strerror or e}')
 
     # Prune empty files
     if files:
@@ -183,8 +177,7 @@ def mkdir(path):
     try:
         os.makedirs(path, exist_ok=True)
     except OSError as e:
-        msg = e.strerror if e.strerror else str(e)
-        raise errors.ContentError(f'{path}: {msg}')
+        raise errors.ContentError(f'{path}: {e.strerror or e}')
     else:
         assert_dir_usable(path)
 
@@ -268,53 +261,70 @@ def tildify_path(path):
     return re.sub(rf'^{re.escape(home)}', '~', path)
 
 
-def file_extension(path):
+def file_extension(path, minlen=1, maxlen=None, only=()):
     """
     Return file extension
 
-    Unlike :func:`os.path.splitext`, this function expects the extension to be 1-3
-    characters long and consist only of ascii characters or numbers.
-
-    :param str path: Path to file or directory
-
-    :return: file extension
-    :rtype: str
-    """
-    filename = os.path.basename(path)
-    match = re.search(r'\.([a-zA-Z0-9]{1,})$', filename)
-    if match:
-        return match.group(1).lower()
-    return ''
-
-def strip_extension(path, only=()):
-    """
-    Return `path` without file extension
-
-    If `path` doesn't have a file extension, return it as is.
-
-    A file extension is 1-3 characters long and consist only of ascii characters
-    or numbers.
+    Unlike :func:`os.path.splitext`, this function expects the extension to
+    consist only of alphanumeric ASCII characters.
 
     :param str path: Path to file or directory
+    :param str minlen: Minimum number of characters in file extension or `None`
+        for no limit
+    :param str maxlen: Maximum number of characters in file extension or `None`
+        for no limit
     :param only: Only strip extension if it exists in this sequence
     :type only: sequence of :class:`str`
 
     :return: file extension
     :rtype: str
     """
+    filename = os.path.basename(path)
+    match = re.search(rf'\.([a-zA-Z0-9]{_make_minmax(minlen, maxlen)})$', filename)
+    if match:
+        ext = match.group(1).lower()
+        if not only or ext in (str(e).lower() for e in only):
+            return ext
+    return ''
+
+def strip_extension(path, minlen=1, maxlen=None, only=()):
+    """
+    Return `path` without file extension
+
+    If `path` doesn't have a file extension, return it as is.
+
+    A file extension consists only of alphanumeric ASCII characters.
+
+    :param str path: Path to file or directory
+    :param str minlen: Minimum number of characters in file extension or `None`
+        for no limit
+    :param str maxlen: Maximum number of characters in file extension or `None`
+        for no limit
+    :param only: Only strip extension if it exists in this sequence
+    :type only: sequence of :class:`str`
+
+    :return: file name without extension
+    :rtype: str
+    """
     path = str(path)
-    if '.' in path:
-        match = re.search(r'^(.*)\.([a-zA-Z0-9]{1,3})$', path)
-        if match:
-            name = match.group(1)
-            ext = match.group(2)
-            if only:
-                if ext in only:
-                    return name
-            else:
-                return name
+    match = re.search(rf'^(.*)\.([a-zA-Z0-9]{_make_minmax(minlen, maxlen)})$', path)
+    if match:
+        name = match.group(1)
+        ext = match.group(2).lower()
+        if not only or ext in (str(e).lower() for e in only):
+            return name
     return path
 
+def _make_minmax(minlen, maxlen):
+    if minlen is not None and maxlen is not None:
+        return '{' + f'{minlen},{maxlen}' + '}'
+    elif minlen is not None:
+        return '{' + f'{minlen},' + '}'
+    elif maxlen is not None:
+        return '{' + f',{maxlen}' + '}'
+    else:
+        return '*'
+
 
 def file_size(path):
     """Return file size in bytes or `None`"""
