@@ -4,7 +4,6 @@ HTML parsing
 
 import re
 
-from .. import errors
 from . import LazyModule
 
 bs4 = LazyModule(module='bs4', namespace=globals())
@@ -18,10 +17,10 @@ def parse(string):
 
     :raise ContentError: if `string` is invalid HTML
     """
-    try:
-        return bs4.BeautifulSoup(str(string), features='html.parser')
-    except Exception as e:
-        raise errors.ContentError(f'Invalid HTML: {e}')
+    if isinstance(string, bs4.element.Tag):
+        return string
+    else:
+        return bs4.BeautifulSoup(string, features='html.parser')
 
 
 def dump(html, filepath):
