@@ -14,8 +14,7 @@ import time
 
 import httpx
 
-from .. import __project_name__, __version__, constants, errors
-from . import fs, html, semantic_hash
+from .. import __project_name__, __version__, constants, errors, utils
 
 import logging  # isort:skip
 _log = logging.getLogger(__name__)
@@ -57,13 +56,15 @@ async def get(
         follow_redirects=True,
         timeout=_default_timeout,
         cookies=None,
+        debug_file=None,
     ):
     """
     Perform HTTP GET request
 
     :param str url: URL to request
     :param dict headers: Custom headers (added to default headers)
-    :param dict params: Query arguments, e.g. `{"k": "v"} → http://foo/bar?k=v`
+    :param dict params: Key value pairs passed in the URL,
+        e.g. ``{"k": "v"}`` → ``http://host/get?k=v``
     :param auth: Basic access authentication; sequence of <username> and
         <password> or `None`
     :param bool cache: Whether to use cached response if available
@@ -98,6 +99,7 @@ async def get(
         follow_redirects=follow_redirects,
         timeout=timeout,
         cookies=cookies,
+        debug_file=debug_file,
     )
 
 async def post(
@@ -112,6 +114,7 @@ async def post(
         follow_redirects=True,
         timeout=_default_timeout,
         cookies=None,
+        debug_file=None,
     ):
     """
     Perform HTTP POST request
@@ -147,10 +150,11 @@ async def post(
         >>>         "mimetype": "text/plain",
         >>>     },
         >>>     "data": {
-        >>>         "file": io.BytesIO(b'binary data'),
+        >>>         "file": io.BytesIO(b"binary data"),
         >>>         "mimetype": "application/octet-stream",
         >>>      },
-        >>> })
+        >>>     "more_data": io.BytesIO(b"more binary data"),
+        >>> }
     :param auth: Basic access authentication; sequence of <username> and
         <password> or `None`
     :param bool cache: Whether to use cached response if available
@@ -178,9 +182,10 @@ async def post(
         follow_redirects=follow_redirects,
         timeout=timeout,
         cookies=cookies,
+        debug_file=debug_file,
     )
 
-async def download(url, filepath, *args, **kwargs):
+async def download(url, filepath, *args, cache=False, **kwargs):
     """
     Write downloaded data to file
 
@@ -195,10 +200,9 @@ async def download(url, filepath, *args, **kwargs):
     :raise RequestError: if anything goes wrong
     :return: `filepath`
     """
-    kwargs['cache'] = False
-    if not os.path.exists(filepath):
+    if not os.path.exists(filepath) or not cache:
         _log.debug('Downloading %r to %r', url, filepath)
-        response = await get(url, *args, **kwargs)
+        response = await get(url, *args, cache=False, **kwargs)
         try:
             with open(filepath, 'wb') as f:
                 f.write(response.bytes)
@@ -210,7 +214,7 @@ async def download(url, filepath, *args, **kwargs):
     return filepath
 
 
-class Result(str):
+class Response(str):
     """
     Response to an HTTP request
 
@@ -276,6 +280,7 @@ async def _request(
         user_agent=False,
         timeout=_default_timeout,
         cookies=None,
+        debug_file=None,
     ):
     if method.upper() not in ('GET', 'POST'):
         raise ValueError(f'Invalid method: {method}')
@@ -299,7 +304,6 @@ async def _request(
             timeout=timeout,
             **build_request_args,
         )
-        _log.debug('Sending headers: %r', request.headers)
 
         # Adjust User-Agent
         if isinstance(user_agent, str):
@@ -319,9 +323,9 @@ async def _request(
                 if result is not None:
                     return result
 
-            _log.debug('Sending request: %r', request)
-            # _log.debug('Request headers: %r', request.headers)
-            # _log.debug('Request data: %r', await request.aread())
+            if debug_file:
+                await _dump_req_res(request, f'{debug_file}.request')
+
             try:
                 response = await client.send(
                     request=request,
@@ -333,7 +337,7 @@ async def _request(
                 except httpx.HTTPStatusError:
                     if response.status_code not in (301, 302, 303, 307, 308):
                         raise errors.RequestError(
-                            f'{url}: {html.as_text(response.text)}',
+                            f'{url}: {utils.html.as_text(response.text)}',
                             url=url,
                             text=response.text,
                             headers=response.headers,
@@ -345,7 +349,7 @@ async def _request(
                 _log.debug(f'Unexpected HTTP error: {e!r}')
                 raise errors.RequestError(f'{url}: {e}')
             else:
-                _save_session_cookies(cookies=response.cookies, domain=response.url.host)
+                _save_session_cookies(cookies=client.cookies, domain=response.url.host)
                 if cookies and isinstance(cookies, (str, pathlib.Path)):
                     _save_permanent_cookies(client=client, filepath=cookies, domain=response.url.host)
 
@@ -353,9 +357,10 @@ async def _request(
                     cache_file = _cache_file(method, url, params)
                     _to_cache(cache_file, response.content)
 
-                # _log.debug('Response content: %r', response.content)
-                # _log.debug('Response headers: %r', response.headers)
-                return Result(
+                if debug_file:
+                    await _dump_req_res(response, f'{debug_file}.response')
+
+                return Response(
                     text=response.text,
                     bytes=response.content,
                     headers=response.headers,
@@ -363,14 +368,41 @@ async def _request(
                 )
 
 
+async def _dump_req_res(r, filepath):
+    content = await r.aread()
+
+    if isinstance(r, httpx.Request):
+        _log.debug('Sending request: %r', r)
+        _log.debug('Request headers: %r', r.headers)
+        _log.debug('Request data: %r', content)
+    else:
+        _log.debug('Received response: %r', r)
+        _log.debug('Response headers: %r', r.headers)
+        _log.debug('Response data: %r', content)
+
+    if isinstance(content, str):
+        utils.html.dump(content, f'{filepath}.html')
+    elif isinstance(content, bytes):
+        with open(f'{filepath}.content', 'wb') as f:
+            f.write(content)
+    elif isinstance(content, str):
+        with open(f'{filepath}.content', 'w') as f:
+            f.write(content)
+    else:
+        raise RuntimeError(f'How should I dump {type(content).__name__}?: {content!r}')
+
+    with open(f'{filepath}.headers', 'w') as f:
+        f.write(json.dumps(dict(r.headers), indent=4))
+
+
 def _load_session_cookies(domain):
-    _log.debug('Loading session cookies for %r: %r', domain, _session_cookies[domain])
+    # _log.debug('Loading session cookies for %r: %r', domain, _session_cookies[domain])
     return _session_cookies[domain]
 
 
 def _save_session_cookies(cookies, domain):
     _session_cookies[domain].update(cookies)
-    _log.debug('Saved session cookies for %r: %r: %r', domain, cookies, _session_cookies[domain])
+    # _log.debug('Saved session cookies for %r: %r: %r', domain, cookies, _session_cookies[domain])
 
 
 def clear_session_cookies(domain=None):
@@ -441,16 +473,28 @@ def _open_files(files):
     """
     opened = {}
     for fieldname, fileinfo in files.items():
+        # Field name is mapped to file path
         if isinstance(fileinfo, str):
             filename = os.path.basename(fileinfo)
             fileobj = _get_file_object(fileinfo)
             opened[fieldname] = (filename, fileobj)
 
+        # Field name is mapped to opened file/file-like object/stream
+        elif isinstance(fileinfo, io.BytesIO):
+            opened[fieldname] = (None, fileinfo)
+
+        # Field name is mapped to dictionary with the keys "file", "filename"
+        # and "mimetype".
+        # - "file" can be a path or file-like object (like above)
+        # - "filename" is the name of the file that is passed to the server or
+        #   `None`.
+        # - "mimetype" is the MIME type, which is autodetected unless "filename"
+        #   is `None`.
         elif isinstance(fileinfo, collections.abc.Mapping):
             if isinstance(fileinfo['file'], str):
                 filename = fileinfo.get('filename', os.path.basename(fileinfo['file']))
                 fileobj = _get_file_object(fileinfo['file'])
-            elif isinstance(fileinfo['file'], io.IOBase):
+            elif isinstance(fileinfo['file'], io.BytesIO):
                 filename = fileinfo.get('filename', None)
                 fileobj = fileinfo['file']
             else:
@@ -490,11 +534,11 @@ def _to_cache(cache_file, bytes):
         except UnicodeDecodeError:
             pass
         else:
-            string = html.purge_javascript(string)
+            string = utils.html.purge_javascript(string)
             bytes = string.encode('utf-8')
 
     try:
-        fs.mkdir(fs.dirname(cache_file))
+        utils.fs.mkdir(utils.fs.dirname(cache_file))
         with open(cache_file, 'wb') as f:
             f.write(bytes)
     except OSError as e:
@@ -512,7 +556,7 @@ def _from_cache(cache_file, max_age=float('inf')):
             bytes = _read_bytes_from_file(cache_file)
             text = _read_string_from_file(cache_file)
             if bytes and text:
-                return Result(text=text, bytes=bytes)
+                return Response(text=text, bytes=bytes)
 
 
 def _read_string_from_file(filepath):
@@ -540,11 +584,11 @@ def _cache_file(method, url, params={}):
     if params:
         params_str = '&'.join((f'{k}={v}' for k,v in params.items()))
         if len(make_filename(method, url, params_str)) > 250:
-            params_str = f'[HASH:{semantic_hash(params)}]'
+            params_str = f'[HASH:{utils.semantic_hash(params)}]'
     else:
         params_str = ''
 
     return os.path.join(
         _get_cache_directory(),
-        fs.sanitize_filename(make_filename(method, url, params_str)),
+        utils.fs.sanitize_filename(make_filename(method, url, params_str)),
     )
