@@ -10,6 +10,7 @@ adds a :class:`~.ReleaseName.format` method to turn everything back into a
 string.
 """
 
+import asyncio
 import collections
 import os
 import re
@@ -18,8 +19,7 @@ import time
 import natsort
 import unidecode
 
-from .. import constants, errors
-from ..utils import iso, scene, webdbs
+from .. import constants, errors, utils
 from . import LazyModule, cached_property, fs, video
 from .types import ReleaseType
 
@@ -114,14 +114,16 @@ class ReleaseName(collections.abc.Mapping):
 
         >>> {
         >>>     'audio_format': {
-        >>>         re.compile(r'^AC-3$'): r'DD',
-        >>>         re.compile(r'^E-AC-3$'): r'DD+',
+        >>>         re.compile(r'^DDP$'): r'DD+',
         >>>     },
         >>>     'video_format': {
         >>>         re.compile(r'^x26([45])$'): r'H.26\1',
         >>>     },
         >>> }
 
+    :param str separator: Separator between release name parts (usually " " or
+        ".") or `None` to use the default
+
     Example:
 
     >>> rn = ReleaseName("path/to/The.Foo.1984.1080p.Blu-ray.X264-ASDF")
@@ -136,16 +138,13 @@ class ReleaseName(collections.abc.Mapping):
     'The Foo (1985) [AsdF]'
     """
 
-    def __init__(self, path, name=None, translate=None):
+    def __init__(self, path, name=None, translate=None, separator=None):
         self._path = str(path)
         self._name = name
+        self.separator = separator
         self._info = ReleaseInfo(str(name) if name is not None else self._path)
         self._translate = translate or {}
 
-    @cached_property
-    def _imdb(self):
-        return webdbs.imdb.ImdbApi()
-
     def set_release_info(self, path):
         """
         Update internal :class:`ReleaseInfo` instance
@@ -188,6 +187,18 @@ class ReleaseName(collections.abc.Mapping):
         else:
             raise KeyError(name)
 
+    @property
+    def separator(self):
+        """
+        Separator between release name parts (usually " " or ".") or `None` to use
+        the default
+        """
+        return self._separator
+
+    @separator.setter
+    def separator(self, separator):
+        self._separator = str(separator) if separator is not None else ' '
+
     @property
     def type(self):
         """
@@ -372,6 +383,20 @@ class ReleaseName(collections.abc.Mapping):
         else:
             self._info['episodes'] = Episodes()
 
+    @property
+    def episodes_dict(self):
+        """Episodes as :class:`Episodes` object"""
+        return self._info.get('episodes') or Episodes()
+
+    @property
+    def only_season(self):
+        """
+        Season if there is only one season in :attr:`episodes_dict`, `None`
+        otherwise
+        """
+        seasons = tuple(self.episodes_dict)
+        return seasons[0] if len(seasons) == 1 else None
+
     @_translated_property
     def episode_title(self):
         """Episode title if :attr:`type` is "episode" or empty string"""
@@ -660,47 +685,45 @@ class ReleaseName(collections.abc.Mapping):
                 return False
         return True
 
-    async def fetch_info(self, *, imdb_id=None, callback=None):
+    async def fetch_info(self, *, webdb, webdb_id, callback=None):
         """
-        Fill in information from IMDb
+        Fill in information web database
 
-        :param str imdb_id: IMDb ID
-        :param callable callback: Function to call after fetching; gets the
-            instance (`self`) as a keyword argument
+        :param webdb: :class:`~.webdbs.base.WebDbApiBase` instance (see
+            :class:`~.webdbs.webdb`)
+        :param str webdb_id: Valid ID for `webdb`
+        :param callable callback: Function to call after info was fetched; gets
+            the instance (`self`) as a keyword argument
 
-        This coroutine function tries to set these attributes:
+        Attempt to set these attributes if they are supported by `webdb`:
 
           - :attr:`title`
           - :attr:`title_aka`
           - :attr:`type`
           - :attr:`year`
-          - :attr:`year_required`
+          - :attr:`year_required` (IMDb only)
           - :attr:`country`
-          - :attr:`country_required`
+          - :attr:`country_required` (IMDb only)
 
         :return: The method's instance (`self`) for convenience
         """
-        # Theoretically, we should use also use `tmdb_id` and `tvmaze_id` to
-        # lookup attributes. Unfortunately, IMDb is the only usable source for
-        # the information we need.
-        for db_name, id in (('_imdb', imdb_id),):
-            db = getattr(self, db_name)
-            await self._update_attributes(db, id)
-        await self._update_type(self._imdb, id)
-        await self._update_year_country_required()
-        _log.debug('Release name updated with IMDb info: %s', self)
-
+        await asyncio.gather(
+            self._update_attributes(webdb, webdb_id),
+            self._update_type(webdb, webdb_id),
+            self._update_country(webdb, webdb_id),
+            self._update_year_country_required(webdb),
+        )
+        _log.debug('Release name updated with %s info: %s', webdb.label, self)
         if callback is not None:
             callback(self)
         return self
 
-    async def _update_attributes(self, db, id):
-        info = await db.gather(
-            id,
+    async def _update_attributes(self, webdb, webdb_id):
+        info = await webdb.gather(
+            webdb_id,
             'title_english',
             'title_original',
             'year',
-            'countries',
         )
         for attr, key in (
             ('title', 'title_original'),
@@ -711,32 +734,41 @@ class ReleaseName(collections.abc.Mapping):
             if info[key]:
                 setattr(self, attr, info[key])
 
-        if info['countries']:
-            self.country = iso.country_tld(info['countries'][0]).upper()
+    async def _update_type(self, webdb, webdb_id):
+        # Use type from database if possible. ReleaseInfo can misdetect (e.g. if
+        # mini-series doesn't contain "S01"). But if ReleaseInfo detected an
+        # episode (e.g. "S04E03"), it's unlikely to be wrong.
+        webdb_type = await webdb.type(webdb_id)
+        if webdb_type and self.type is not ReleaseType.episode:
+            self.type = webdb_type
 
-    async def _update_type(self, db, id):
-        # Use type from database if possible. ReleaseInfo can misdetect type
-        # (e.g. if mini-series doesn't contain "S01"). But if ReleaseInfo
-        # detected an episode (e.g. "S04E03"), it's unlikely to be wrong.
-        db_type = await db.type(id)
-        if db_type and self.type is not ReleaseType.episode:
-            self.type = db_type
+    async def _update_country(self, webdb, webdb_id):
+        try:
+            countries = await webdb.countries(webdb_id)
+        except NotImplementedError:
+            pass
+        else:
+            if countries:
+                self.country = utils.country.tld(countries[0]).upper()
 
-    async def _update_year_country_required(self):
+    async def _update_year_country_required(self, webdb):
         if self.type in (ReleaseType.season, ReleaseType.episode):
+            # Find out if there are any TV show name duplicates. If so, require
+            # year and/or country code in release name to pin it down.
+
             def normalize_title(title):
                 return unidecode.unidecode(title.casefold())
 
             # Find result with the same title, removing any "smart" matches
             title_normalized = normalize_title(self.title)
-            query = webdbs.Query(title=self.title, type=ReleaseType.series)
+            query = utils.webdbs.Query(title=self.title, type=ReleaseType.series)
             results = [
                 {
                     'title': normalize_title(result.title),
                     'year': result.year,
                     'countries': await result.countries(),
                 }
-                for result in await self._imdb.search(query)
+                for result in await webdb.search(query)
                 if title_normalized == normalize_title(result.title)
             ]
 
@@ -761,7 +793,7 @@ class ReleaseName(collections.abc.Mapping):
                 elif not has_duplicates(make_title(r, country=True) for r in results):
                     self.country_required = True
 
-    def format(self, sep=' '):
+    def format(self, separator=None):
         """Assemble all parts into string"""
         parts = [self.title_with_aka_and_year]
 
@@ -785,7 +817,9 @@ class ReleaseName(collections.abc.Mapping):
             parts.append(self.audio_channels)
         parts.append(self.video_format)
 
-        return sep.join(parts) + f'-{self.group}'
+        joined = ' '.join(parts) + f'-{self.group}'
+        sep = separator if separator is not None else self._separator
+        return joined.replace(' ', sep)
 
 
 class ReleaseInfo(collections.abc.MutableMapping):
@@ -832,7 +866,7 @@ class ReleaseInfo(collections.abc.MutableMapping):
     def __init__(self, path, strict=False):
         if strict:
             try:
-                scene.assert_not_abbreviated_filename(path)
+                utils.scene.assert_not_abbreviated_filename(path)
             except errors.SceneAbbreviatedFilenameError as e:
                 raise errors.ContentError(e)
         self._path = str(path)
@@ -885,6 +919,8 @@ class ReleaseInfo(collections.abc.MutableMapping):
     def _guess(self):
         path = self._abspath
 
+        # TODO: Remove when this is fixed upstream:
+        #       https://github.com/guessit-io/guessit/pull/733
         # guessit doesn't detect AC3 if it's called "AC-3"
         path = re.sub(rf'({DELIM})(?i:AC-?3)({DELIM})', r'\1AC3\2', path)
         path = re.sub(rf'({DELIM})(?i:E-?AC-?3)({DELIM})', r'\1EAC3\2', path)
@@ -917,7 +953,7 @@ class ReleaseInfo(collections.abc.MutableMapping):
             if (
                 'S' in episodes_string
                 and 'E' not in episodes_string
-                and scene.is_abbreviated_filename(path)
+                and utils.scene.is_abbreviated_filename(path)
             ):
                 filename = fs.basename(path)
                 match = re.search(r'((?i:[SE]\d+)+)', filename)
@@ -1019,7 +1055,7 @@ class ReleaseInfo(collections.abc.MutableMapping):
     }
 
     def _get_country(self):
-        country = iso.country_tld(
+        country = utils.country.tld(
             _as_string(self._guess.get('country') or '')
         ).upper()
         for country_, regex in self._country_translation.items():
@@ -1114,7 +1150,7 @@ class ReleaseInfo(collections.abc.MutableMapping):
     _streaming_service_regex = re.compile(rf'{DELIM}([A-Z]+){DELIM}(?i:WEB-?(?:DL|Rip))(?:{DELIM}|$)')
     _streaming_service_translation = {
         re.compile(r'(?i:IT)'): 'iT',
-        re.compile(r'(?i:ATVP)'): 'APTV',
+        re.compile(r'(?i:4OD)'): 'ALL4',   # 4oD is old name of Channel 4's VOD service
         # Not a streaming service
         re.compile(r'OAR'): '',  # Original Aspect Ratio
     }
@@ -1177,18 +1213,13 @@ class ReleaseInfo(collections.abc.MutableMapping):
             if match:
                 source = match.group(1)
 
-        elif source == 'DVD':
-            if 'Rip' in self._guess.get('other', ()):
-                source = 'DVDRip'
-            else:
-                if 'DVD9' in self.release_name_params:
-                    source = 'DVD9'
-                elif 'DVD5' in self.release_name_params:
-                    source = 'DVD5'
+        elif 'Rip' in self._guess.get('other', ()):
+            source = f'{source}Rip'
 
-        elif source == 'TV':
-            if 'Rip' in self._guess.get('other', ()):
-                source = 'TVRip'
+        elif 'DVD9' in self.release_name_params:
+            source = 'DVD9'
+        elif 'DVD5' in self.release_name_params:
+            source = 'DVD5'
 
         if source:
             # Fix spelling
@@ -1207,10 +1238,10 @@ class ReleaseInfo(collections.abc.MutableMapping):
         return source
 
     _audio_codec_translation = {
-        re.compile(r'^AC-?3')             : 'AC-3',
-        re.compile(r'Dolby Digital$')     : 'AC-3',
-        re.compile(r'^E-?AC-?3')          : 'E-AC-3',
-        re.compile(r'Dolby Digital Plus') : 'E-AC-3',
+        re.compile(r'^AC-?3')             : 'DD',
+        re.compile(r'Dolby Digital$')     : 'DD',
+        re.compile(r'^E-?AC-?3')          : 'DDP',
+        re.compile(r'Dolby Digital Plus') : 'DDP',
         re.compile(r'TrueHD')             : 'TrueHD',
         re.compile(r'Dolby Atmos')        : 'Atmos',
         re.compile(r'Master Audio')       : 'MA',
@@ -1238,13 +1269,12 @@ class ReleaseInfo(collections.abc.MutableMapping):
                         parts.append(abbrev)
                         continue
 
-            if parts:
-                audio_codec = ' '.join(parts)
-                audio_codec = re.sub(r'(?:E-|)AC-?3 TrueHD', 'TrueHD', audio_codec)
-                audio_codec = re.sub(r'(?:E-|)AC-?3 Atmos', 'Atmos', audio_codec)
-            else:
-                # Codecs like "MP3" or "FLAC" are already abbreviated
-                audio_codec = ' '.join(infos)
+            # Join translations, default to what guessit detected
+            audio_codec = ' '.join(parts or infos)
+
+            # Final adjustments
+            audio_codec = re.sub(r'DDP? TrueHD', 'TrueHD', audio_codec)
+            audio_codec = re.sub(r'DDP? Atmos', 'Atmos', audio_codec)
 
             return audio_codec
 
