@@ -6,7 +6,7 @@ import abc
 import asyncio
 import copy
 
-from .. import iso
+from .. import country
 from .common import Query
 
 
@@ -76,7 +76,7 @@ class WebDbApiBase(abc.ABC):
     async def countries(self, id):
         """Return list of country names"""
         countries = await self._countries(id)
-        return iso.country_name(countries)
+        return country.name(countries)
 
     @abc.abstractmethod
     async def _countries(self, id):
@@ -96,7 +96,14 @@ class WebDbApiBase(abc.ABC):
 
     @abc.abstractmethod
     async def poster_url(self, id):
-        """Return URL of poster image or `None`"""
+        """
+        Return URL of poster image or `None`
+
+        :param season: Return poster for specific season
+
+            If this is not supported by the concrete implementation, default to
+            the URL for the main poster.
+        """
 
     @abc.abstractmethod
     async def rating(self, id):
@@ -123,7 +130,7 @@ class WebDbApiBase(abc.ABC):
         """
         runtimes = {}
         for key, runtime in (await self._runtimes(id)).items():
-            key = iso.country_name(key)
+            key = country.name(key)
             if key not in runtimes and runtime not in runtimes.values():
                 runtimes[key] = runtime
         return runtimes
