@@ -28,6 +28,8 @@ class ImdbApi(WebDbApiBase):
     _url_base = 'https://www.imdb.com'
     _soup_cache = {}
 
+    user_agent = 'Mozilla/5.0 (Windows NT 10.0; rv:107.0) Gecko/20100101 Firefox/107.0'
+
     async def _get_soup(self, path, params={}):
         cache_id = (path, tuple(sorted(params.items())))
         if cache_id in self._soup_cache:
@@ -35,6 +37,7 @@ class ImdbApi(WebDbApiBase):
         text = await utils.http.get(
             url=f'{self._url_base}/{path}',
             params=params,
+            user_agent=self.user_agent,
             cache=True,
         )
         self._soup_cache[cache_id] = utils.html.parse(text)
@@ -93,10 +96,18 @@ class ImdbApi(WebDbApiBase):
             if query.year is not None:
                 params['release_date'] = f'{query.year}-01-01,{query.year}-12-31'
 
+            def is_search_result(tag):
+                if tag.get('class') == ['lister-item-content']:
+                    header = tag.find(class_='lister-item-header')
+                    if not header or 'Episode:' not in header.strings:
+                        return True
+
             soup = await self._get_soup(path, params=params)
-            items = soup.find_all('div', class_='lister-item-content')
-            results = [_ImdbSearchResult(soup=item, imdb_api=self)
-                       for item in items]
+            items = soup.find_all(is_search_result)
+            results = [
+                _ImdbSearchResult(soup=item, imdb_api=self)
+                for item in items
+            ]
             return results
 
     _person_url_path_regex = re.compile(r'(/name/nm\d+)')
@@ -181,7 +192,7 @@ class ImdbApi(WebDbApiBase):
             return tuple(genres)
         return ()
 
-    async def poster_url(self, id):
+    async def poster_url(self, id, season=None):
         if id:
             soup = await self._get_soup(f'title/{id}/')
             a_tag = soup.find(href=re.compile(r'/title/tt\d+/mediaviewer/'))
@@ -330,15 +341,37 @@ class ImdbApi(WebDbApiBase):
         bn = self._normalize_title(b)
         return an and bn and (an in bn or bn in an)
 
-    _normalize_title_translation = str.maketrans('', '', string.punctuation)
-
     def _normalize_title(self, title):
-        """Return casefolded `title` without punctuation and deduplicated whitespace"""
+        """Return casefolded `title` normalized punctuation, whitespace, etc"""
+        # Replace special characters with similar ASCII
         title = title.replace('&', 'and')
         title = unidecode.unidecode(title)
-        title = title.translate(self._normalize_title_translation)
-        title = title.casefold()
+
+        # Remove all punctuation
+        title = re.sub(rf'[{string.punctuation}]+', '', title)
+
+        # Deduplicate whitespace into spaces (U+0020)
+        title = ' '.join(title.split())
+
+        # Remove difference between arabic and roman numbers
+        def normalize_part(match):
+            num = match.group(1)
+            if num.isdigit():
+                num_arabic = num
+                num_roman = int(num) * 'I'
+            else:
+                num_arabic = len(num)
+                num_roman = num
+            return f'{num_arabic}/{num_roman}'
+
+        title = re.sub(r'\b((?i:I+|\d+))\b', normalize_part, title)
+
+        # Remove all whitespace
         title = ''.join(title.split())
+
+        # Case-insensitivize
+        title = title.casefold()
+
         return title
 
     _ignored_akas_keys = (
