@@ -149,7 +149,7 @@ class TmdbApi(WebDbApiBase):
                           if k != ',')
         return tuple(genres)
 
-    async def poster_url(self, id):
+    async def poster_url(self, id, season=None):
         if id:
             soup = await self._get_soup(id)
             img_tag = soup.find('img', class_='poster')
